;ͻ
;Communication routines                                                       
;ͼ

A_FAUSTUS       = 0
A_GOAT          = 1
A_PHANTASM      = 2
A_HITMAN        = 3
A_DETECTIVE     = 4
A_MUSICSHOP     = 5
A_WEAPONSHOP    = 6
A_BAR           = 7

K_SADOK         = 0
K_HITMEN        = 1
K_JOIN          = 2
K_MURDER        = 3
K_CAVES         = 4
K_PLOT          = 5
K_WHAT          = 6
K_ABD           = 7
K_EVD           = 8

;Ŀ
;INITCOMM                                                                     
;                                                                             
;Clears the communication keyword table in the beginning of the game          
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

initcomm:       lda #$00
                sta keywordnum
                tax
ico_loop:       sta actorkeytbl,x
                inx
                cpx #MAX_ACTOR*16
                bcc ico_loop
                rts

;Ŀ
;CHECKCOMM                                                                    
;                                                                             
;Checks for going into communication mode                                     
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

checkcomm:      jsr isdead                      ;Requirements: player alive,
                bne com_ok1                     ;spacebar pressed, player not
                rts                             ;jumping, screen not scrolling
com_ok1:        lda keyval
                cmp #KEY_SPACE
                beq com_ok2
                rts
com_ok2:        lda shpbits
                ora scradd
                beq com_ok3
                rts
com_ok3:        ldx #FIRSTNPC
com_loop:       lda shpt,x
                cmp #FCHARACTER                 ;Is it an actor that can be
                bcc com_next                    ;communicated with?
                cmp #LCHARACTER+1
                bcs com_next
                lda shpxl,x                     ;Check for being too far away
                sec                             ;In X-dir
                sbc shpxl
                sta alo
                lda shpxh,x
                sbc shpxh
                sta ahi
                bmi com_xdneg
                bne com_next
                lda alo
                cmp #$20
                bcs com_next
                bcc com_xpass
com_xdneg:      cmp #$ff
                bne com_next
                lda alo
                cmp #$e0
                bcc com_next
com_xpass:      lda shpyl,x                     ;Then in Y-dir
                sec
                sbc shpyl
                sta alo
                lda shpyh,x
                sbc shpyh
                sta ahi
                bmi com_ydneg
                bne com_next
                lda alo
                cmp #$20
                bcs com_next
                bcc comm
com_ydneg:      cmp #$ff
                bne com_next
                lda alo
                cmp #$e0
                bcs comm
com_next:       inx
                cpx #LASTNPC+1
                bne com_loop
                rts

;Ŀ
;COMM                                                                         
;                                                                             
;The communication routine itself.                                            
;                                                                             
;Parameters: X:shape index of the character to be communicated with           
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

comm:           lda shpt,x                      ;Take the actor-index of the
                sec                             ;character
                sbc #FCHARACTER
                sta actornum
                jsr clearsprites
                lda actornum                    ;Is it a "shopkeeper" or
                cmp #MAX_ACTOR                  ;"conversation" actor?
                bcc com_plrtalk
                jmp shop
com_plrtalk:    lda knowdetails                 ;Update keyword status for the
                beq com_dontknow                ;hitman character based on
                lda actorkeytbl+A_FAUSTUS*$10+K_MURDER ;whether player talked
                sta actorkeytbl+A_HITMAN*$10+K_MURDER ;with FAUSTUS first
com_dontknow:   jsr com_drawwindow              ;Draw communication window with
                ldx #$00                        ;player face onscreen
                jsr com_setsprite
                lda #<talktotxt
                sta textlo
                lda #>talktotxt
                sta texthi
                ldx #$06
                ldy #$06
                jsr printtext
                lda actornum                    ;Find actor name
                asl
                tax
                lda actornametbl,x
                sta textlo
                lda actornametbl+1,x
                sta texthi
                jsr pt_mulready
                lda #<abouttxt
                sta textlo
                lda #>abouttxt
                sta texthi
                jsr pt_mulready
                lda #$e8                        ;INX - search forwards
                sta com_findkeydir
com_findkey:    lda actornum                    ;Find a keyword that is valid
                asl                             ;with this actor
                asl
                asl
                asl
                sta alo
                adc keywordnum
                tax
                ldy #$10
com_findkeyloop:lda actorkeytbl,x
                bne com_keyfound
com_findkeydir: inx
                txa
                and #$0f
                ora alo
                tax
                dey
                bne com_findkeyloop
com_nokeys:     jsr com_drawwindow              ;If no keywords, display
                lda #<nokeytxt                  ;sarcastic message
                sta textlo
                lda #>nokeytxt
                sta texthi
                ldx #$06
                ldy #$06
                jsr printtext
                jsr com_fireorspace
                jsr com_waitnofire
                jmp com_quit
com_keyfound:   txa
                and #$0f
                sta keywordnum
com_printkey:   ldx #$06                        ;Keyword found, display it
                ldy #$07
                jsr printtext_getpos
                lda #$20
                ldy #29
com_pkloop:     sta (scrolltemp1),y
                dey
                bpl com_pkloop
                lda keywordnum
                asl
                tax
                lda keywordtbl,x
                sta textlo
                lda keywordtbl+1,x
                sta texthi
                jsr pt_mulready
com_wait:       jsr com_waitshit                ;Loop for selecting keyword
                lda prevjoy
                bne com_nojoy
                lda joystick
                and #JOY_LEFT+JOY_UP
                bne com_left
                lda joystick
                and #JOY_RIGHT+JOY_DOWN
                bne com_right
                lda joystick
                and #JOY_FIRE
                bne com_fire
com_nojoy:      lda keyval                      ;SPACE quits conversation at
                cmp #KEY_SPACE                  ;any point
                bne com_wait
                jmp com_quit
com_right:      lda keywordnum
                clc
                adc #$01
                and #$0f
                sta keywordnum
                lda #$e8                        ;INX - forwards
                sta com_findkeydir
                jmp com_findkey
com_left:       lda keywordnum
                sec
                sbc #$01
                and #$0f
                sta keywordnum
                lda #$ca                        ;DEX - backwards
                sta com_findkeydir
                jmp com_findkey
com_fire:       jsr com_getactortbl             ;Fire button pressed -
                lda keywordnum                  ;conversation proceeds
                asl
                tay
                lda (alo),y                     ;Take corresponding keyword
                sta com_jump1+1                 ;code address from jump table
                iny
                lda (alo),y
                sta com_jump1+2
                ldx #$01                        ;Put the NPC's face to the
                jsr com_setsprite               ;screen
                ldx #<com_nothing
                ldy #>com_nothing               ;Second jump does nothing by
                jsr com_setjump2                ;default; but keyword specific
com_jump1:      jsr $0000                       ;code can change this
                stx textlo
                sty texthi
com_print3lines:jsr com_drawwindow              ;Print max. 3 rows of text
                lda #$06
                sta alo
com_printline:  ldy alo
                ldx #$06
                jsr printtext
                ldy #$00
                lda (textlo),y
                cmp #$ff
                beq com_jump2
                inc alo
                lda alo
                cmp #$09
                bne com_printline
                jsr com_fireorspace
                bcc com_quit
                bcs com_print3lines
com_jump2:      jsr $0000
                jsr com_fireorspace
                bcc com_quit
                jmp com_plrtalk
com_quit:       jsr clearsprites                ;Exiting the conversation
                jmp drawscreen

;Ŀ
;SHOP                                                                         
;                                                                             
;Buying-routine                                                               
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

shop:           lda #$00
                sta shopitemnum
shop_sfx:       lda #SFX_SELECT
                jsr playsfx
shop_loop:      jsr waitraster
                jsr com_drawwindow
                jsr com_getactortbl
                jsr com_getitemnum
                sta selitem
                iny
                lda (alo),y
                sta selitemcost
                iny
                lda (alo),y
                sta selitempower
                lda selitem
                ora #$80
                tax
                jsr com_setsprite
                lda selitem
                clc
                adc #MSG_ITEM1
                asl
                tax
                lda msgptr,x
                sta textlo
                lda msgptr+1,x
                sta texthi
                ldx #$06
                ldy #$06
                jsr printtext
                lda selitempower
                cmp #$02
                bcc shop_skippower
                and #$0f
                clc
                adc #$30
                sta roundtext+3
                lda selitempower
                lsr
                lsr
                lsr
                lsr
                clc
                adc #$30
                sta roundtext+2
                lda #<roundtext
                sta textlo
                lda #>roundtext
                sta texthi
                jsr pt_mulready
shop_skippower: lda selitemcost
                and #$0f
                clc
                adc #$30
                sta costtext+6
                lda selitemcost
                lsr
                lsr
                lsr
                lsr
                clc
                adc #$30
                sta costtext+5
                lda #<costtext
                sta textlo
                lda #>costtext
                sta texthi
                ldx #$06
                ldy #$08
                jsr printtext
shop_wait:      jsr com_waitshit
                lda prevjoy
                bne shop_nojoy
                lda joystick                    ;Selecting item to buy
                and #JOY_LEFT+JOY_UP
                bne shop_left
                lda joystick
                and #JOY_RIGHT+JOY_DOWN
                bne shop_right
                lda joystick
                and #JOY_FIRE
                bne shop_fire
shop_nojoy:     lda keyval
                cmp #KEY_SPACE
                bne shop_wait
                jmp com_quit
shop_right:     inc shopitemnum
                jsr com_getactortbl
                jsr com_getitemnum
                cmp #$ff
                beq shop_right1
                jmp shop_loop
shop_right1:    lda #$00
                sta shopitemnum
                jmp shop_loop
shop_left:      dec shopitemnum
                bmi shop_left1
                jmp shop_loop
shop_left1:     lda #$08
                sta shopitemnum
shop_left2:     dec shopitemnum
                jsr com_getactortbl
                jsr com_getitemnum
                cmp #$ff
                beq shop_left2
                jmp shop_loop
shop_fire:      lda #ITEM_CREDITS               ;Fire button pressed: check
                jsr finditem                    ;player's credits situation
                bcc shop_nomoney
                lda plrinvp,y
                sty moneyidx
                cmp selitemcost
                bcc shop_nomoney
                lda selitem                     ;Check for room in inventory
                sta ahi
                jsr checkempty
                bcc shop_noroom
                lda plrinvp,y                   ;Check for full ammo
                cmp #$99
                beq shop_fullammo
                lda ahi
                sta plrinvt,y
                lda plrinvp,y
                sed
                clc
                adc selitempower                ;Add ammo
                bcs shop_over
                cmp #$99
                bcc shop_notover
shop_over:      lda #$99
shop_notover:   cld
                sta plrinvp,y
                jsr updatepower
                ldx #100
                lda #MSG_BOUGHT
                jsr newmsg
                ldy moneyidx                    ;Subtract credits
                lda plrinvp,y
                sed
                sec
                sbc selitemcost
                cld
                sta plrinvp,y
                bne shop_notzero
                lda #$80
                sta plrinvt,y
shop_notzero:   jsr updatepower
                jmp shop_sfx
shop_nomoney:   lda #MSG_NOMONEY
                jsr pickup_no2
                jmp shop_loop
shop_noroom:    jsr pickup_no
                jmp shop_loop
shop_fullammo:  lda #MSG_FULLAMMO
                jsr pickup_no2
                jmp shop_loop

        ;Waiting-subroutine. Returns Carry=1 on pressing fire and Carry=0 on
        ;pressing space.

com_fireorspace:jsr com_waitshit
                lda prevjoy
                bne com_nojoy2
                lda joystick
                and #JOY_FIRE
                bne com_fireexit
com_nojoy2:     lda keyval
                cmp #KEY_SPACE
                bne com_fireorspace
                clc
                rts
com_fireexit:   sec
com_nothing:    rts
com_waitnofire: jsr com_waitshit
                lda joystick
                and #JOY_FIRE
                bne com_waitnofire
                rts

com_waitshit:   jsr waitraster
                jsr updatescore
                jsr getcontrols
                jmp selectsfx

com_setjump2:   stx com_jump2+1
                sty com_jump2+2
                rts

com_addscore:   lda #$00
                sta alo
                lda #$20
                sta ahi
                jmp addscore
com_addscore2:  ldx #$05
com_addscore2l: jsr com_addscore
                dex
                bne com_addscore2l
                rts

com_getactortbl:lda actornum
                asl
                tax
                lda actortbl,x
                sta alo
                lda actortbl+1,x
                sta ahi
                rts

com_getitemnum: lda shopitemnum
                asl
                adc shopitemnum
                tay
                lda (alo),y
                rts

com_setsprite:  lda #$01
                sta numspr
                lda #$30
                clc
                adc scrollx
                sta sprx
                lda #98
                adc scrolly
                sta spry
                lda #$00
                sta sprfls
                sta sprxh
                lda #236
                cpx #$00
                bmi com_item
                beq com_ssplr
                sec
                adc actornum
com_ssplr:      sta sprf
                lda #SFX_SELECT
                jsr playsfx
com_sprdone:    jsr updatesprites
                jsr waitraster
                inc newsprites
                rts
com_item:       txa
                and #$7f
                clc
                adc itemframetbl
                sta sprf
                jmp com_sprdone

com_drawwindow: ldx screen
                lda #243
                sta scrolltemp1
                sta scrolltemp3
                lda scradrtblhi,x
                sta scrolltemp2
                lda #>COLORS
                sta scrolltemp4
                ldx #$03
com_dwloop1:    ldy #$00
com_dwloop2:    lda #41
                sta (scrolltemp1),y
                lda #$06
                sta (scrolltemp3),y
                iny
                cpy #$03
                bne com_dwloop2
com_dwloop3:    lda #$20
                sta (scrolltemp1),y
                lda #$01
                sta (scrolltemp3),y
                iny
                cpy #34
                bne com_dwloop3
                lda scrolltemp1
                clc
                adc #40
                sta scrolltemp1
                sta scrolltemp3
                bcc com_dwnotover
                inc scrolltemp2
                inc scrolltemp4
com_dwnotover:  dex
                bne com_dwloop1
                rts

fa_sadok:       ldx #<fa_sethitmen              ;Here begins the keyword
                ldy #>fa_sethitmen              ;specific code (quite messy :-))
                jsr com_setjump2
                ldx #<fa_sadoktxt
                ldy #>fa_sadoktxt
                rts
fa_sethitmen:   lda actorkeytbl+A_FAUSTUS*$10+K_HITMEN
                bne fa_rts
                lda #$01
                sta actorkeytbl+A_FAUSTUS*$10+K_HITMEN
                sta actorkeytbl+A_GOAT*$10+K_HITMEN
                sta actorkeytbl+A_PHANTASM*$10+K_HITMEN
                lda #K_HITMEN
                sta keywordnum
                jmp com_addscore
fa_hitmen:      ldx #<fa_hitmentxt
                ldy #>fa_hitmentxt
fa_rts:         rts
fa_join:        lda #ITEM_GUITAR
                jsr finditem
                bcc fa_jnogtr
                ldx #<endepisode1
                ldy #>endepisode1
                jsr com_setjump2
                ldx #<fa_jointxt2
                ldy #>fa_jointxt2
                rts
fa_jnogtr:      ldx #<fa_jointxt1
                ldy #>fa_jointxt1
                rts
fa_murder:      ldx #<fa_murdertxt
                ldy #>fa_murdertxt
                rts
fa_caves:       ldx #<fa_cavestxt
                ldy #>fa_cavestxt
                rts
fa_plot:        ldx #<fa_plottxt
                ldy #>fa_plottxt
                rts
fa_what:        ldx #<fa_setabd
                ldy #>fa_setabd
                jsr com_setjump2
                ldx #<fa_whattxt
                ldy #>fa_whattxt
                rts
fa_setabd:      lda #$00
                sta actorkeytbl+A_FAUSTUS*$10+K_WHAT
                sta actorkeytbl+A_GOAT*$10+K_WHAT
                lda #$01
                sta actorkeytbl+A_FAUSTUS*$10+K_ABD
                sta actorkeytbl+A_GOAT*$10+K_ABD
                lda #MSG_KNOW
                ldx #160
                jsr newmsg
                jsr com_addscore
                jsr com_fireorspace
                jsr com_waitnofire
                pla
                pla
                jmp com_quit
fa_abd:         ldx #<fa_abdtxt
                ldy #>fa_abdtxt
                rts
fa_evd:         ldx #<fa_evdtxt
                ldy #>fa_evdtxt
                rts

go_sadok:       ldx #<go_sadoktxt
                ldy #>go_sadoktxt
                rts
go_hitmen:      ldx #<go_hitmen2
                ldy #>go_hitmen2
                jsr com_setjump2
                ldx #<go_hitmentxt
                ldy #>go_hitmentxt
                rts
go_hitmen2:     lda actorkeytbl+A_HITMAN*$10+K_HITMEN
                bne go_rts
                lda #$01
                sta actorkeytbl+A_HITMAN*$10+K_HITMEN
                jmp com_addscore
go_join:        ldx #<go_jointxt
                ldy #>go_jointxt
go_rts:         rts
go_murder:      ldx #<go_murdertxt
                ldy #>go_murdertxt
                rts
go_caves:       ldx #<go_cavestxt
                ldy #>go_cavestxt
                rts
go_plot:        ldx #<go_plottxt
                ldy #>go_plottxt
                rts
go_abd:         ldx #<go_abdtxt
                ldy #>go_abdtxt
                rts
go_evd:         ldx #<go_evdtxt
                ldy #>go_evdtxt
                rts

ph_sadok:       ldx #<ph_setjoin
                ldy #>ph_setjoin
                jsr com_setjump2
                ldx #<ph_sadoktxt
                ldy #>ph_sadoktxt
                rts
ph_setjoin:     lda actorkeytbl+A_PHANTASM*$10+K_JOIN
                bne ph_rts
                lda #$01
                sta actorkeytbl+A_FAUSTUS*$10+K_JOIN
                sta actorkeytbl+A_GOAT*$10+K_JOIN
                sta actorkeytbl+A_PHANTASM*$10+K_JOIN
                lda #K_JOIN
                sta keywordnum
                jmp com_addscore
ph_hitmen:      ldx #<ph_hitmentxt
                ldy #>ph_hitmentxt
ph_rts:         rts
ph_join:        ldx #<ph_thought
                ldy #>ph_thought
                jsr com_setjump2
                ldx #<ph_jointxt
                ldy #>ph_jointxt
ph_rts2:        rts
ph_thought:     lda #ITEM_GUITAR
                jsr finditem
                bcs ph_rts2
                lda #MSG_THOUGHT2
                ldx #200
                jmp newmsg
ph_murder:      ldx #<ph_murdertxt
                ldy #>ph_murdertxt
                rts
ph_caves:       ldx #<ph_cavestxt
                ldy #>ph_cavestxt
                rts
ph_plot:        ldx #<ph_plottxt
                ldy #>ph_plottxt
                rts

dm_hitmen:      lda givecard
                bne dm_getlost
                lda #ITEM_BEER
                jsr finditem
                bcc dm_burp
                jsr decammo2
                lda #$01
                sta knowdetails
                ldx #<dm_hitmentxt
                ldy #>dm_hitmentxt
                rts
dm_burp:        ldx #<dm_burptxt
                ldy #>dm_burptxt
                rts
dm_murder:      lda givecard
                bne dm_getlost
                lda #ITEM_BEER
                jsr finditem
                bcc dm_burp
                jsr decammo2
                ldx #<dm_murder2
                ldy #>dm_murder2
                jsr com_setjump2
                ldx #<dm_murdertxt
                ldy #>dm_murdertxt
                rts
dm_murder2:     inc givecard
                jsr com_addscore
                lda #27
                sta LEVELWORK+$3f
                lda #19
                sta LEVELWORK+$3f+$40
                lda #IT+ITEM_KEYCARD1
                sta LEVELWORK+$3f+$80
                lda #$01
                sta LEVELWORK+$3f+$c0
                jsr com_fireorspace
                jsr com_waitnofire
                pla
                pla
                jmp com_quit
dm_getlost:     ldx #<dm_getlosttxt
                ldy #>dm_getlosttxt
                rts

dh_sadok:       ldx #<dh_sadoktxt
                ldy #>dh_sadoktxt
                rts
dh_murder:      ldx #<dh_murdertxt
                ldy #>dh_murdertxt
                rts
dh_plot:        ldx #<dh_plottxt
                ldy #>dh_plottxt
                rts
dh_evd:         lda #ITEM_EVIDENCE
                jsr finditem
                bcc dh_evd2
                ldx #<dh_givecard
                ldy #>dh_givecard
                jsr com_setjump2
                ldx #<dh_evdtxt
                ldy #>dh_evdtxt
                rts
dh_evd2:        ldx #<dh_evdtxt2
                ldy #>dh_evdtxt2
                rts
dh_givecard:    jsr com_addscore
                lda #ITEM_EVIDENCE
                jsr finditem
                bcc dh_gcskip
                jsr decammo2
dh_gcskip:      lda #16
                sta LEVELWORK+$3f
                lda #17
                sta LEVELWORK+$3f+$40
                lda #IT+ITEM_KEYCARD2
                sta LEVELWORK+$3f+$80
                lda #$01
                sta LEVELWORK+$3f+$c0
                jsr com_fireorspace
                jsr com_waitnofire
                pla
                pla
                jmp com_quit



