;ͻ
;Inventory & scoreroutines                                                    
;ͼ

;Ŀ
;INITITEMS                                                                    
;                                                                             
;Clears player inventory at the start of the game.                            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

inititems:      ldy #$07
ii_loop:        lda #$80
                sta plrinvt,y
                lda #$00
                sta plrinvp,y
                jsr updatepower
                dey
                bpl ii_loop
                lda #$00
                sta itemnum
                jsr paintinv
                rts

;Ŀ
;SELECTITEM                                                                   
;                                                                             
;Selection of active item with keyboard.                                      
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

selectitem:     lda keyval
                cmp #KEY_X
                beq si_next
                cmp #KEY_PISTE
                bne si_notnext
si_next:        lda itemnum
                cmp #$07
                beq si_quit
                inc itemnum
si_doit:        lda #SFX_SELECT
                jsr playsfx
                jsr paintinv
si_quit:        rts
si_notnext:     cmp #KEY_Z
                beq si_prev
                cmp #KEY_PILKKU
                bne si_quit
si_prev:        lda itemnum
                beq si_quit
                dec itemnum
                jmp si_doit

;Ŀ
;PAINTINV                                                                     
;                                                                             
;Paints active inventory-item with different color than the rest.             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                 
;Modifies: A,X,Y                                                              
;

paintinv:       ldx #$15
                ldy #$07
pi_loop:        lda #$00
                cpy itemnum
                bne pi_notsel
                lda #$06
pi_notsel:      sta COLORS+20*40+1,x
                sta COLORS+20*40+2,x
                sta COLORS+21*40+1,x
                sta COLORS+21*40+2,x
                dex
                dex
                dex
                dey
                bpl pi_loop
                rts

;Ŀ
;UPDATEPOWER                                                                  
;                                                                             
;Updates ammo in inventory.                                                   
;                                                                             
;Parameters: Y:item number                                                    
;Returns: -                                                                   
;Modifies: A,X                                                                
;
updatepower:    tya
                sta scrolltemp1
                asl
                adc scrolltemp1
                tax
                lda plrinvp,y
                lsr
                lsr
                lsr
                lsr
                clc
                adc #48
                sta SCREEN1+22*40+1,x
                lda plrinvp,y
                and #$0f
                clc
                adc #48
                sta SCREEN1+22*40+2,x
                rts

;Ŀ
;FINDITEM                                                                     
;                                                                             
;Searches inventory for a certain item.                                       
;Parameters: A:item type                                                      
;Returns: Carry=1 if item found, index in Y-register                          
;Modifies: A,Y                                                                
;

finditem:       ldy #$07
                sta find_cmp+1
find_loop:      lda plrinvt,y
find_cmp:       cmp #$00
                beq find_found
                dey
                bpl find_loop
                clc
                rts
find_found:     sec
                rts

;Ŀ
;PICKUP                                                                       
;                                                                             
;Checks for items to be picked up at player's feet. If there is, picks them   
;up.                                                                          
;                                                                             
;Parameters: X:Player actor index (0)                                         
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

pickup:         ldy #FIRSTNPC
pickup_check:   lda shpt,y
                cmp #SHP_ITEM                   ;Is an item?
                bne pickup_next
                jsr collide                     ;Collided with player?
                bcs pickup_yes
pickup_next:    iny                             ;Went thru all actors to be
                cpy #LASTNPC+2                  ;checked?
                bne pickup_check
                rts
pickup_yes:     lda shpf,y
                sta ahi
                sty alo                         ;Store item number
                jsr checkempty                  ;Is there room?
                bcc pickup_no
                ldx alo                         ;Item number to X
                lda ahi
                sta plrinvt,y
                pha
                lda plrinvp,y
                sed
                clc
                adc shpp,x                      ;Add ammo
                bcs pickup_over
                cmp #$99
                bcc pickup_notover
pickup_over:    lda #$99
pickup_notover: cld
                sta plrinvp,y
                lda #$00                        ;Remove item from the ground
                sta shpt,x
                jsr updatepower
                pla
                clc
                adc #MSG_ITEM1
                ldx #25
                jsr newmsg
                lda #SFX_SELECT
                jsr playsfx
                rts
pickup_no:      lda #MSG_NOROOM
pickup_no2:     ldx #100
                jsr newmsg
                lda #SFX_HIT
                jsr playsfx
                rts

;Ŀ
;CHECKEMPTY                                                                   
;                                                                             
;Checks for room in player inventory.                                         
;                                                                             
;Parameters: -                                                                
;Returns: Carry=1 if there is room, Y is the free inventoryindex              
;Modifies: A,Y                                                                
;

checkempty:     ldy #$00
ce_loop1:       lda plrinvt,y                   ;Check for the same item
                cmp ahi
                beq ce_found
                iny
                cpy #$08
                bne ce_loop1
                ldy itemnum                     ;Check for free location at
                lda plrinvt,y                   ;the currently selected item
                bmi ce_found
                ldy #$00
ce_loop2:       lda plrinvt,y                   ;If this fails, go thru all
                bmi ce_found                    ;inventory slots
                iny
                cpy #$08
                bne ce_loop2
ce_noroom:      clc
                rts
ce_found:       cmp #ITEM_BEER
                beq ce_found1
                cmp #ITEM_HEALTH                ;Cap max. carried first aid kits
                bne ce_found2                   ;and beer
ce_found1:      lda plrinvp,y
                cmp #MAX_MEDIKITS
                bcs ce_noroom
ce_found2:      sec
                rts

;Ŀ
;ADDSCORE                                                                     
;                                                                             
;Adds alo,ahi to player score.                                                
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

addscore:       jsr addscore_nl
                sed
                lda score+2
                cmp extracmp
                bcc adsc_nolives
                lda extracmp
                clc
                adc #$05
                sta extracmp
                inc lives
                lda lives
                cmp #$0a
                bcc adsc_nolives
                lda #$09
                sta lives
adsc_nolives:   cld
                rts

addscore_nl:    sed
                lda score
                clc
                adc alo
                sta score
                lda score+1
                adc ahi
                sta score+1
                lda score+2
                adc #0
                sta score+2
                lda score+3
                adc #0
                sta score+3
                cld
                rts

;Ŀ
;UPDATESCORE                                                                  
;                                                                             
;Updates score, energy and lives in the scorepanel.                           
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

updatescore:    lda score                       ;No update if the value is same
                cmp prevscore
                bne us_notsame1
                lda score+1
                cmp prevscore+1
                bne us_notsame1
                lda score+2
                cmp prevscore+2
                bne us_notsame1
                lda score+3
                cmp prevscore+3
                beq us_skip1
us_notsame1:    ldx #3
                ldy #0
us_loop:        lda score,x
                sta prevscore,x
                lsr
                lsr
                lsr
                lsr
                clc
                adc #48
                sta SCREEN1+20*40+31,y
                iny
                lda score,x
                and #$0f
                clc
                adc #48
                sta SCREEN1+20*40+31,y
                iny
                dex
                bpl us_loop
us_skip1:       lda lives
                clc
                adc #48
                sta SCREEN1+22*40+28
                lda shpp
                cmp prevpower
                beq us_edone
                sta prevpower
                lsr                     ;Divide energy by four to get amount of
                lsr                     ;full chars
                ldy #$00
                tax
                beq us_energy2
                lda #61
us_energy1:     sta SCREEN1+22*40+31,y
                iny
                dex
                bne us_energy1
                cpy #8                  ;Ready yet?
                beq us_edone
us_energy2:     lda shpp
                and #$fc
                sta alo
                lda shpp                ;Then take the remaining part
                sec
                sbc alo
                beq us_energy3          ;If zero, print nothing
                adc #57-1
                sta SCREEN1+22*40+31,y
                iny
us_energy3:     cpy #8                  ;Fill rest with empty space
                beq us_edone            ;(energy meter total length is 8 chars)
                lda #32
                sta SCREEN1+22*40+31,y
                iny
                bne us_energy3
us_edone:       rts

;Ŀ
;RANDOM                                                                       
;                                                                             
;Quite fast pseudorandom number generator                                     
;(uses code as a lookup table, performs additions)                            
;                                                                             
;Parameters: -                                                                
;Returns: A - "random" number                                                 
;Modifies: A                                                                  
;

random:         inc randomadd+1
                bne random2
                inc randomadd+2
                lda randomadd+2
                cmp #$28
                bcc random2
                lda #$20
                sta randomadd+2
random2:        lda #$00
randomadd:      eor $2000
                adc #$3d
                sta random2+1
                rts


