;ͻ
;Move routines except for enemies                                             
;ͼ

;Ŀ
;GETFIRECLICK                                                                 
;                                                                             
;Checks for the first frame where a firebutton is pressed                     
;                                                                             
;Parameters: -                                                                
;Returns: Carry=1 if pressed                                                  
;Modifies: A                                                                  
;

getfireclick:   lda joystick
                and #JOY_FIRE
                beq gfc_not
                lda prevjoy
                and #JOY_FIRE
                bne gfc_not
                sec
                rts
gfc_not:        clc
                rts

;Ŀ
;GETCONTROLS                                                                  
;                                                                             
;Takes controls from joystickport 2                                           
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

getcontrols:    lda #$ff
                sta $dc00
                lda joystick
                sta prevjoy
                lda $dc00
                eor #$ff
                sta joystick

;Ŀ
;SCANKEYS                                                                     
;                                                                             
;Reads keyboard. Ripped from Jukka Tapanimki's C64 game programming guide.   
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

scankeys:       lda #$ff
                sta keyval
                ldy #$07
sk_rowloop:     lda rowbit,y
                sta $dc00
                ldx $dc01
                cpx #$ff
                bne sk_keyfound
                dey
                bpl sk_rowloop
                lda #$ff
                sta lastkey
                rts
sk_keyfound:    sty keytemp
                ldy #7
sk_getcol:      txa
                and colbit,y
                beq sk_colfound
                dey
                bpl sk_getcol
sk_colfound:    ldx keytemp
                tya
                clc
                adc rowval,x
                cmp lastkey
                beq sk_samekey
                sta keyval
                sta lastkey
sk_samekey:     rts

;Ŀ
;INITPLAYER                                                                   
;INITNEXTLIFE                                                                 
;                                                                             
;Initializes player in different ways. INITPLAYER resets lives & score,       
;INITNEXTLIFE inits energy.                                                   
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

initplayer:     lda #$00
                ldx #prevscore-score-1
initplayerloop: sta score,x
                dex
                bpl initplayerloop
                sta shpd
                lda #$05
                sta extracmp
                lda #$03
                sta lives
                jmp inl_maxpower
initnextlife:   lda #SHP_PLRRUN
                sta shpt
                ldx #$ff
                stx shporg
                inx
                stx shpf
                stx shpfd
                stx shpsy
                stx shpbits
                stx firedelay
                stx drunk
                lda #150                ;Immortality for 3 seconds
                sta shpimm
inl_maxpower:   lda #PLR_MAXPOWER
                sta shpp
                rts

;Ŀ
;CHECKDEATH                                                                   
;                                                                             
;Initializes new life/ends game when out of lives.                            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

isdead:         lda shpt
                beq dead
                cmp #SHP_PLRDEAD
dead:           rts

checkdeath:     jsr isdead
                beq chd_noskip
chd_notyet:     rts
chd_noskip:     inc deadcnt             ;Wait 200 frames after death
                lda deadcnt
                cmp #200
                bcc chd_notyet
chd_livesdec:   dec lives               ;Decrease lives
                beq chd_gameover        ;Go to gameover if out of them
                jsr leveltune_nocheck
                jsr initnextlife
                lda lastxl
                sta shpxl
                lda lastxh
                sta shpxh
                lda lastyl
                sta shpyl
                lda lastyh
                sta shpyh
                lda lastt
                sta shpt
                rts
chd_gameover:   pla                     ;Jump directly out of the main loop
                pla
                jmp gameover


;Ŀ
;MOVPLR                                                                       
;                                                                             
;Player move routines (running, jumping, ducking, climbing)                   
;                                                                             
;Parameters: X:actor number (0)                                               
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

movplr_drunk:   lda drunk
                beq movplr_notdrunk
                dec drunkd
                bpl movplr_nodd
                lda #$08                        ;If player character has
                sta drunkd                      ;drunk alcohol, use random
                dec drunk                       ;controls
                jsr random
                and #JOY_LEFT+JOY_RIGHT
                sta drunkrnd
movplr_nodd:    lda joystick
                beq movplr_notdrunk
                and #JOY_UP+JOY_DOWN+JOY_FIRE
                ora drunkrnd
                sta joystick
movplr_notdrunk:rts

movplr_run:     jsr movplr_drunk
                lda shpbits,x                   ;Player jumping?
                and #$01
                beq movplr_nofly
                jmp movplr_fly
movplr_nofly:   jsr getcharpos                  ;Get char position
                jsr getcharinfo
                tay
                and #CI_WALK                    ;Feet under ground?
                bne movplr_nofall               ;If not, fall
                lda #2
                sta shpsy,x
                jmp movplr_fall
movplr_nofall:  lda joystick                    ;Ducking?
                and #JOY_DOWN
                beq movplr_noduck
                tya                             ;Ladder?
                and #CI_CLIMB
                bne movplr_initclimb
                lda #SHP_PLRDUCK
                sta shpt,x
                jsr movecharposup1              ;Drop guitar next to an
                jsr getcharinfo                 ;amplifier? (weird feature)
                and #$10
                beq movplr_dnogtr
                lda #ITEM_GUITAR
                jsr finditem
                bcc movplr_dnogtr
                jsr decammo2
                ldx #$00
movplr_gtrloop: lda lvlshpt,x
                beq movplr_gtrempty
                inx
                cpx #NUM_LVLSHP
                bcc movplr_gtrloop
                jmp movplr_nopickup
movplr_gtrempty:lda #SHP_GUITAR
                sta lvlshpt,x
                lda #$01
                sta lvlshpp,x
                lda charmapx
                sta lvlshpx,x
                lda charmapy
                clc
                adc #$01
                sta lvlshpy,x
                jmp movplr_nopickup
movplr_dnogtr:  jsr pickup                      ;Picking up an item?
movplr_nopickup:ldx #$00
                txa
                sta shpf,x
                rts
movplr_noduck:  lda joystick                    ;Initiate jump?
                and #JOY_UP
                beq movplr_nojump
                jsr movecharposup1
                jsr getcharinfo                 ;Go inside a door?
                sta scrolltemp3
                dec charmapy
                jsr getcharinfo                 ;Ladder?
                and #CI_CLIMB
                php
                jsr movecharposdown1
                inc charmapy
                plp
                beq movplr_noclup
movplr_initclimb:
                lda shpxl,x                     ;Set X-position in the middle
                and #$e0                        ;of the ladder
                clc
                adc #$10
                sta shpxl,x
                lda shpyl,x                     ;Round Y-position to an even
                and #$fe                        ;number
                sta shpyl,x
                lda #SHP_PLRCLIMB
                sta shpt,x
                lda #$00
                sta shpf,x
                sta shpfd,x
                rts
movplr_noclup:  lda prevjoy                     ;Prevent continuous jumping
                and #JOY_UP
                bne movplr_nojump
                lda scrolltemp3                 ;Door?
                and #CI_DOOR
                beq movplr_nodoor
                jmp doorsearch
movplr_nodoor:  lda #SFX_JUMP
                jsr playsfx
                lda #-2                         ;Init jump
                sta shpsy,x
movplr_fall:    lda #0
                sta shpfd,x
                lda #7
                sta shpf,x
                lda shpbits,x                   ;Flying bit on
                ora #$01
                sta shpbits,x
                rts
movplr_nojump:  lda joystick                    ;Moving left?
                and #JOY_LEFT
                beq movplr_notleft
                lda #1
                sta shpd,x
                jsr movecharposleft1
                jsr movecharposup1
                jsr getcharinfo
                and #CI_OBSTACLE                ;Can player go left?
                bne movplr_nomove
                ldy #-2
                jsr moveshpx
                jsr checkstairs
                jmp movplr_anim
movplr_notleft: lda joystick                    ;Moving right?
                and #JOY_RIGHT
                beq movplr_nomove
                lda #0
                sta shpd,x
                jsr movecharposright1
                jsr movecharposup1
                jsr getcharinfo
                and #CI_OBSTACLE                ;Can player go right?
                bne movplr_nomove
                ldy #2
                jsr moveshpx
                jsr checkstairs
                jmp movplr_anim
movplr_nomove:  lda #6                          ;No movement, set "standing"
                sta shpf,x                      ;frame
                lda #3
                sta shpfd,x
                rts
movplr_anim:    lda #$00
                inc shpfd,x                     ;Running animation
                ldy shpfd,x
                cpy #5
                bcc movplr_done                 ;New frame on every fifth
                sta shpfd,x                     ;frame
                inc shpf,x
                ldy shpf,x
                cpy #6
                bcc movplr_anot
                sta shpf,x
                tay
movplr_anot:    lda walksndtbl,y                ;Footstep sound on every third
                beq movplr_done                 ;frame
                jsr playsfx
movplr_done:    rts

movplr_duck:    jsr movplr_drunk
                lda joystick
                and #JOY_LEFT
                beq movplr_dnotl
                lda #1
                sta shpd,x
                jmp movplr_dcheck
movplr_dnotl:   lda joystick
                and #JOY_RIGHT
                beq movplr_dcheck
                lda #0
                sta shpd,x
movplr_dcheck:  lda joystick
                and #JOY_DOWN
                bne movplr_done
                lda #SHP_PLRRUN
                sta shpt,x
                jmp movplr_nomove

movplr_fly:     jsr movplr_drunk
                inc shpfd,x
                ldy shpsy,x                     ;Motion in Y-direction
                jsr moveshpy
                jsr getcharpos                  ;Take charposition now
                ldy shpsy,x                     ;Going up or down?
                bpl movplr_fdown
                dec charmapy                    ;Move check position 4 chars up
                jsr getcharinfo
                inc charmapy
                and #CI_OBSTACLE                ;Check obstacle at player's head
                beq movplr_fnohead
                lda #0                          ;If an obstacle, zero y-speed
                sta shpsy,x                     ;(head bumped to ceiling)
                sta shpfd,x
                jmp movplr_fdown
movplr_fnohead: lda joystick                    ;Player holding joystick up to
                and #JOY_UP                     ;prolong jump?
                beq movplr_fdown
                lda shpfd,x
                cmp #13
                bcc movplr_fno1
                jmp movplr_fincspd
movplr_fdown:   lda shpfd,x
                cmp #7
                bcc movplr_fno1
movplr_fincspd: lda #0
                sta shpfd,x
                inc shpsy,x
                lda shpsy,x
                bmi movplr_fno1
                cmp #3                          ;Y-speed must not be >2
                bcc movplr_fno1
                lda #2
                sta shpsy,x
movplr_fno1:    lda shpsy,x                     ;Check ground only if Y-speed
                bmi movplr_fside                ;> 0
                beq movplr_fside
                jsr getcharinfo                 ;Ground under feet?
                and #CI_WALK
                bne movplr_flgrsfx
movplr_fside:   lda joystick                    ;Check for horizontal movement
                and #JOY_LEFT
                beq movplr_fnotl
                lda #1
                sta shpd,x
                jsr movecharposleft1
                jsr getcharinfo
                and #CI_OBSTACLE
                bne movplr_fgc
                ldy #-2
                jsr moveshpx
                jmp movplr_fgc
movplr_fnotl:   lda joystick
                and #JOY_RIGHT
                beq movplr_fgc
                lda #0
                sta shpd,x
                jsr movecharposright1
                jsr getcharinfo
                and #CI_OBSTACLE
                bne movplr_fgc
                ldy #2
                jsr moveshpx

movplr_fgc:     jsr getcharpos
                lda shpsy,x
                bmi movplr_fngc
                beq movplr_fngc
                jsr getcharinfo
                and #CI_WALK
                beq movplr_fngc
movplr_flgrsfx: lda #SFX_FOOTSTEP
                jsr playsfx
movplr_flground:lda shpyl,x                     ;Player "landed", and Y-coord
                and #$f8                        ;to "round" it on char boundary
                sta shpyl,x
                lda shpbits,x                   ;Zero the jumping bit
                and #$fe
                sta shpbits,x
                lda #0
                sta shpsy,x                     ;Zero Y-speed
                jmp movplr_nomove               ;Set standing position
movplr_fngc:    rts

movplr_climb:   jsr getcharpos
                lda joystick
                and #JOY_UP
                bne movplr_cup
                lda joystick
                and #JOY_DOWN
                bne movplr_cdown
                lda joystick                  ;Exit ladder to right?
                and #JOY_RIGHT
                beq movplr_crdone
                lda #0
                sta shpd,x
                jsr getcharinfo
                and #CI_WALK
                beq movplr_crdone
                lda #SHP_PLRRUN
                sta shpt,x
                jmp movplr_flground
movplr_crdone:  lda joystick                ;Exit ladder to left?
                and #JOY_LEFT
                beq movplr_cdone
                lda #1
                sta shpd,x
                jsr getcharinfo
                and #CI_WALK
                beq movplr_cdone
                lda #SHP_PLRRUN
                sta shpt,x
                jmp movplr_flground
movplr_cup:     dec charmapy
                jsr movecharposup1
                jsr getcharinfo
                and #CI_CLIMB
                beq movplr_cdone
                ldy #-1
                jsr moveshpy
                jmp movplr_canim
movplr_cdown:   lda joystick
                and #JOY_DOWN
                beq movplr_cdone
                jsr getcharinfo
                and #CI_CLIMB
                beq movplr_cdone
                ldy #1
                jsr moveshpy
movplr_canim:   inc shpfd,x                     ;Climbing animation
                lda shpfd,x
                cmp #4
                bcc movplr_cdone
                lda #0
                sta shpfd,x
                lda shpf,x
                clc
                adc #1
                and #3
                sta shpf,x
movplr_cdone:   rts

movplr_dead:    inc shpfd,x
                lda shpfd,x
                cmp #$05
                bne movplr_dead1
                lda #$00
                sta shpfd,x
                lda shpsy,x
                bmi movplr_dead2
                cmp #$04
                bcs movplr_dead1
movplr_dead2:   inc shpsy,x
movplr_dead1:   ldy shpsy,x
                jsr moveshpy
                rts

checkstairs:    jsr getcharpos                  ;Walking into emptiness?
                jsr getcharinfo                 ;teen?
                and #CI_WALK
                bne stnot2                      ;No, exit this routine!
                jsr movecharposup1              ;Check for stairs above and
                jsr getcharinfo                 ;below
                and #CI_WALK
                beq stnot1
                ldy #-8
                jsr moveshpy
                clc
                rts
stnot1:         ldy #2
                jsr movecharposdown
                jsr getcharinfo
                and #CI_WALK
                beq stempty
                ldy #8                          ;Return carry 1 if no stairs
                jsr moveshpy                    ;found (initiate falling)
stnot2:         clc
                rts
stempty:        sec
                rts

;Ŀ
;MOVESHPX, MOVESHPY                                                           
;                                                                             
;Moves an actor.                                                              
;                                                                             
;Parameters: X:Actor number                                                   
;            Y:Amount of movement in pixels                                   
;Returns: -                                                                   
;Modifies: A                                                                  
;

moveshpx:       tya
                clc
                bmi mspx_neg
                adc shpxl,x
                sta shpxl,x
                lda shpxh,x
                adc #0
                sta shpxh,x
                rts
mspx_neg:       adc shpxl,x
                sta shpxl,x
                lda shpxh,x
                adc #$ff
                sta shpxh,x
                rts

moveshpy:       tya
                clc
                bmi mspy_neg
                adc shpyl,x
                sta shpyl,x
                lda shpyh,x
                adc #0
                sta shpyh,x
                rts
mspy_neg:       adc shpyl,x
                sta shpyl,x
                lda shpyh,x
                adc #$ff
                sta shpyh,x
donothing:      rts

;Ŀ
;ITEMROUT                                                                     
;                                                                             
;Item's move routine (falling)                                                
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

itemrout:       ldy shpsy,x
                beq itr_nomove
                jsr moveshpy
                inc shpfd,x
                lda shpfd,x
                cmp #$04
                bcc itr_not
                lda #$00
                sta shpfd,x
                inc shpsy,x
                lda shpsy,x
                cmp #$04
                bcc itr_not
                lda #$04
                sta shpsy,x
itr_not:        jsr getcharpos
                jsr getcharinfo
                and #CI_WALK
                bne itr_ground
                rts
itr_ground:     lda #$00
                sta shpsy,x
                lda shpyl,x
                and #$f8
                sta shpyl,x
itr_nomove:     rts

;Ŀ
;EXPLODE                                                                      
;                                                                             
;Explosion animation routine.                                                 
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

explode:        lda #$05
                sta explcheck+1
explode2:       inc shpfd,x
                lda shpfd,x
                cmp #$03
                bne exp_not
                lda #$00
                sta shpfd,x
                inc shpf,x
                ldy shpf,x
explcheck:      cpy #$05
                bne exp_not
                sta shpt,x
exp_not:        rts
dblexpl:        lda #$07
                sta explcheck+1
                bne explode2

;Ŀ
;SHRAPNEL                                                                     
;                                                                             
;Shrapnel animation.                                                          
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

shrapnel:       inc shpfd,x
                lda shpfd,x
                cmp #3
                bne shr_not
                lda #0
                sta shpfd,x
                inc shpf,x
                ldy shpf,x
                cpy #3
                bne shr_not
                sta shpt,x
shr_not:        rts

;Ŀ
;BLOOD                                                                        
;                                                                             
;Animation and movement of blood                                              
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

blood:          ldy #1
                jsr moveshpy
                inc shpfd,x
                lda shpfd,x
                cmp #3
                bne bld_not
                lda #0
                sta shpfd,x
                inc shpf,x
                ldy shpf,x
                cpy #2
                bne shr_not
                sta shpt,x
bld_not:        rts

;Ŀ
;SIGN                                                                         
;                                                                             
;Flashing arrow animation (indicates important places to visit)               
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

sign:           inc shpfd,x
                lda shpfd,x
                cmp #$10
                bcc sign_not
sign_anim:      lda #$00
                sta shpfd,x
                lda shpf,x
                eor #$01
                sta shpf,x
sign_not:       rts

;Ŀ
;HARRIS                                                                       
;                                                                             
;Steve Harris-imitation of FAUSTUS (fingers animation)                        
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

harris:         lda episode
                cmp #$03
                beq harris_epi3
                bcs harris_epi4
                inc shpfd,x
                lda shpfd,x
                cmp #$04
                bcc harris_not
                lda #$00
                sta shpfd,x
                lda shpf,x
                adc #$00
                and #$07
                sta shpf,x
harris_not:     rts
harris_epi3:    lda shpyl,x
                ora #$10
                sta shpyl,x
harris_epi4:    lda #$08
                sta shpf,x
                rts

;Ŀ
;OVERMAN                                                                      
;                                                                             
;GOAT's smoking animation                                                     
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

overman:        lda episode
                cmp #$03
                beq om_epi3
                bcs om_epi4
                inc shpfd,x
                lda shpfd,x
                cmp #$05
                bcc harris_not
                jmp sign_anim
om_epi3:        lda shpxl,x
                ora #$1e
                sta shpxl,x
om_epi4:        lda #$02
                sta shpf,x
                rts

;Ŀ
;YNGWIE                                                                       
;                                                                             
;PHANTASM's Yngwie scale practicing                                           
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                 
;Modifies: A,Y                                                                
;

yngwie:         lda episode
                cmp #$03
                beq ym_epi3
                inc shpfd,x
                lda shpfd,x
                cmp #$06
                bcc yngwie_not
                lda #$00
                sta shpfd,x
                jsr random
                and #$01
                bne yngwie_up
yngwie_down:    lda shpf,x
                beq yngwie_up2
yngwie_down2:   dec shpf,x
                rts
yngwie_up:      lda shpf,x
                cmp #$03
                bcs yngwie_down2
yngwie_up2:     inc shpf,x
yngwie_not:     rts
ym_epi3:        lda #$04
                sta shpf,x
                lda shpyl,x
                ora #$10
                sta shpyl,x
                lda shpxl,x
                ora #$1c
                sta shpxl,x
                rts

;Ŀ
;SADOK                                                                        
;                                                                             
;SADOK in the cave                                                            
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                 
;Modifies: A,Y                                                                
;

sadok:          lda shpf,x
                bne sadok_anim
                lda shpxl,x
                sec
                sbc shpxl
                sta alo
                lda shpxh,x
                sbc shpxh
                sta ahi
                bne sadok_not
                lda alo
                cmp #$60
                bcs sadok_not
                lda #$01
                sta shpf,x
                stx alo
                lda #MSG_SADOK
                ldx #120
                jsr newmsg
                ldx alo
                lda #$00
                sta shpsy,x
sadok_not:      rts
sadok_anim:     inc shpfd,x
                lda shpfd,x
                cmp #$06
                bcc sadok_not
                lda #$00
                sta shpfd,x
                lda shpf,x
                cmp #$06
                bcs sadok_animdone
                inc shpf,x
                rts
sadok_animdone: inc shpsy,x
                lda shpsy,x
                cmp #20
                bcc sadok_not
                lda #$ff
                sta deadbosstype
                rts

;Ŀ
;MOVBULLET                                                                    
;                                                                             
;Bullet move routine.                                                         
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

movbullet:      jsr getcharpos                  ;Get char position
                jsr getcharinfo
                sta alo
                and #CI_OBSTACLE                ;Bullet collided with BG?
                beq mb_noobst
                lda shpf,x
                cmp #3                          ;Grenade explodes, other turn
                beq explodeshape                ;into the shrapnel/smokecloud
                lda #SHP_SHRAPNEL               ;animation
                sta shpt,x
                bne explshp2
mb_grenexpl:    jsr blast
explodeshape:   lda #SFX_EXPLOSION
                jsr playsfx
                lda #SHP_EXPLOSION
                sta shpt,x
explshp2:       lda #0
                sta shpfd,x
                sta shpf,x
                rts
mb_noobst:      lda shpsx,x                     ;Perform X-movement
                ldy shpd,x
                beq mb_right
                eor #$ff
                clc
                adc #$01
mb_right:       tay
                jsr moveshpx
                lda shpf,x
                cmp #$03                        ;Handling of grenades
                beq mb_grenade
                php
                ldy shpsy,x
                jsr moveshpy
                plp
                bcs mb_flame
                rts
mb_grenade:     lda #CI_OBSTACLE
                sta ahi
                lda shpsy,x
                bmi mb_grenup
                lda #CI_OBSTACLE+CI_WALK
                sta ahi
mb_grenup:      lda alo
                and ahi
                bne mb_grenexpl
                jsr getcharpos                  ;Check BG collision two times
                jsr getcharinfo                 ;to prevent going through from
                and ahi                         ;corners
                bne mb_grenexpl
                ldy shpsy,x
                inc shpfd,x
                lda shpfd,x
                cmp #5
                bne mb_grnot
                lda #0
                sta shpfd,x
                iny
                bmi mb_grnot
                cpy #4
                bcc mb_grnot
                ldy #4
mb_grnot:       tya
                sta shpsy,x
                jsr moveshpy
                rts
mb_flame:       inc shpfd,x                     ;Flame animation
                lda shpfd,x
                cmp #4
                bne mb_fnotnext
                lda #0
                sta shpfd,x
                inc shpf,x
                ldy shpf,x
                cpy #8
                bne mb_fnotnext
                sta shpt,x
mb_fnotnext:    rts

;Ŀ
;SHOOT                                                                        
;                                                                             
;Player's shooting & item use routine                                         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

decammo:        ldy itemnum
decammo2:       sed
                lda plrinvp,y                   ;Decrease ammo. When ammo is
                sec                             ;zero, remove item from
                sbc #$01                        ;inventory.
                sta plrinvp,y
                cld
                jsr updatepower
                lda plrinvp,y
                bne dec_notzero
                lda #$80
                sta plrinvt,y
dec_notzero:    rts

sh_beer:        lda prevjoy
                and #JOY_FIRE
                bne sh_hquit
                lda #$20
                sta drunk
                lda #$08
                sta drunkd
                jsr decammo
                lda shpp
                clc
                adc #$04
                jmp sh_healthcheck

sh_health:      lda prevjoy
                and #JOY_FIRE
                bne sh_hquit
                lda shpp
                cmp #PLR_MAXPOWER
                bcs sh_hquit
                jsr decammo
                lda shpp
                clc
                adc #$0c
sh_healthcheck: cmp #PLR_MAXPOWER
                bcc sh_health_no
                lda #PLR_MAXPOWER
sh_health_no:   sta shpp
                lda #$08
                sta shpimm
                lda #SFX_SELECT
                jsr playsfx
sh_hquit:       rts

shoot:          lda firedelay
                beq sh_ok1
                dec firedelay
sh_ok1:         jsr isdead                      ;Player alive?
                beq sh_quit
                lda joystick                    ;Firebutton pressed?
                and #JOY_FIRE
                beq sh_quit
                ldy itemnum
                lda plrinvt,y                   ;Weapon selected?
                cmp #ITEM_BEER
                beq sh_beer
                cmp #ITEM_HEALTH
                beq sh_health
                bcs sh_quit
                sta alo
                lda firedelay
                bne sh_quit
                lda shpt                        ;No shooting while climbing
                cmp #SHP_PLRCLIMB
                bne sh_ok2
sh_quit:        rts
sh_ok2:         ldx #FIRSTPLRBLT
sh_loop:        lda shpt,x
                beq sh_ok3
                inx
                cpx #LASTPLRBLT+1
                bne sh_loop
                rts
sh_ok3:         lda alo                         ;Is it the pistol?
                beq sh_ammook                   ;(unlimited ammo)
                stx ahi
                jsr decammo
                ldx ahi
sh_ammook:      ldy alo
                lda wpnsfx,y
                jsr playsfx
                lda wpnbltframe,y
                sta shpf,x
                lda wpndelay,y
                sta firedelay
                lda wpnsx,y
                sta shpsx,x
                lda #0
                sta shpfd,x
                lda wpnsy,y
                sta shpsy,x
                lda shpd
                tay
                sta shpd,x
                lda shpxl
                clc
                adc wpnaddxlo,y
                sta shpxl,x
                lda shpxh
                adc wpnaddxhi,y
                sta shpxh,x
                ldy shpt
                lda shpyl
                sec
                sbc wpnaddy-1,y
                sta shpyl,x
                lda shpyh
                sbc #0
                sta shpyh,x
                lda #SHP_BULLET
                sta shpt,x
noshoot:        rts

;Ŀ
;ENEMYSHOOT                                                                   
;                                                                             
;Enemy shooting routine                                                       
;                                                                             
;Parameters: X:Enemy actor number                                             
;            A:Weapon number                                                  
;Returns: Carry=1 indicates successful shooting                               
;Modifies: A,Y                                                                
;

enemyshoot:     sta alo
                ldy #FIRSTENMBLT
esh_loop:       lda shpt,y
                beq esh_ok
                iny
                cpy #LASTENMBLT+1
                bne esh_loop
                clc                             ;Carry 0
                rts
esh_ok:         stx ahi
                ldx alo
                lda wpnsfx,x
                jsr playsfx
                lda wpnbltframe,x
                sta shpf,y
                lda wpnsx,x
                sta shpsx,y
                lda wpnsy,x
                sta shpsy,y
                lda #0
                sta shpfd,y
                ldx ahi
                lda shpt,x
                sec
                sbc #FENEMYTYPE
                tax
                lda enemysmodx,x
                sta esh_addx+1
                sta esh_subx+1
                lda #$00
                sta esh_subyhi+1
                lda enemysmody,x
                sta esh_suby+1
                bpl esh_notneg
                dec esh_subyhi+1
esh_notneg:     ldx ahi
                lda shpd,x
                sta shpd,y
                bne esh_left
esh_right:      lda shpxl,x
                clc
esh_addx:       adc #$00
                sta shpxl,y
                lda shpxh,x
                adc #$00
                sta shpxh,y
                jmp esh_xdone
esh_left:       lda shpxl,x
                sec
esh_subx:       sbc #$00
                sta shpxl,y
                lda shpxh,x
                sbc #$00
                sta shpxh,y
esh_xdone:      lda shpyl,x
                sec
esh_suby:       sbc #$00
                sta shpyl,y
                lda shpyh,x
esh_subyhi:     sbc #$00
                sta shpyh,y
                lda #SHP_BULLET
                sta shpt,y
                sec
                rts

;Ŀ
;DAMAGE                                                                       
;                                                                             
;Damages player character, killing if health goes to zero.                    
;                                                                             
;Parameters: ALO:amount of damage                                             
;Returns: -                                                                   
;Modifies: A                                                                  
;

damage:         jsr isdead                      ;Do not touch already-dead or
                beq dm_immortal                 ;immortals :-)
                lda shpimm
                bne dm_immortal
                lda #SFX_HIT
                jsr playsfx
                lda #$04
                sta shpimm
                lda shpp
                sec
                sbc alo
                bcc dm_kill
                beq dm_kill
                sta shpp
dm_immortal:    rts
dm_kill:        lda shpxl
                sta lastxl
                lda shpxh
                sta lastxh
                lda shpyl
                sta lastyl
                lda shpyh
                sta lastyh
                lda shpt
                sta lastt
                lda #$00
                sta shpp
                sta shpf
                sta shpfd
                sta shpbits
                sta deadcnt
                lda #SHP_PLRDEAD
                sta shpt
                lda #-3
                sta shpsy
                lda #MUS_DEATH
                jsr playtune
                rts

;Ŀ
;CHECKSCROLL                                                                  
;                                                                             
;Checks for player going close to edges of screen and scrolling if necessary. 
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

checkscroll:    jsr isdead      ;If dead, no scrolling
                bne chs_noskip
                lda #$00
                sta scrollsx
                sta scrollsy
                rts
chs_noskip:     ldx #$00                        ;Assume: no X-scrolling
                lda shpxl
                sec
                sbc scrposxl
                sta scrolltemp1
                lda shpxh
                sbc scrposxh
                sta scrolltemp2
                lda scrolltemp2
                bne chscr_over255
                lda scrolltemp1
                cmp #241
                bcc chscr_notright
                ldx #1
                jmp chscr_xdone
chscr_over255:  ldx #1
                lda scrolltemp1
                cmp #1
                bcc chscr_xdone
                ldx #2
                jmp chscr_xdone
chscr_notright: cmp #128
                bcs chscr_xdone
                ldx #-1
                cmp #112
                bcs chscr_xdone
                ldx #-2
chscr_xdone:    stx scrollsx
                ldx #0                          ;Assume: no Y-scrolling
                lda level
                cmp #LVL_CITY                   ;In the city level, no
                beq chscr_ydone                 ;Y-scrolling at all
                lda shpyl
                sec
                sbc scrposyl
                cmp #174
                bcc chscr_notdown
                ldx #1
                cmp #182
                bcc chscr_ydone
                ldx #2
                jmp chscr_ydone
chscr_notdown:  cmp #128
                bcs chscr_ydone
                ldx #-1
                cmp #120
                bcs chscr_ydone
                ldx #-2
chscr_ydone:    stx scrollsy
                rts

;Ŀ
;WARPMAP                                                                      
;                                                                             
;Moves to another location in the game world.                                 
;                                                                             
;Parameters: X,Y:block position, A:level number                               
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

warpmap:        stx warpdx
                sty warpdy
                sta warpdl
                lda #$01
                sta warp
                rts

;Ŀ
;DOORSEARCH                                                                   
;                                                                             
;Searches for a door at player's position and performs entering the door.     
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

doorsearch:     lda bossfight     ;Doors don't "work" while in a boss-fight
                bne dsr_quit
                ldy #NDOORS-1
                dec charmapy
dsr_loop:       lda charmapx
                cmp doorx,y
                bne dsr_skip
                lda charmapy
                cmp doory,y
                bne dsr_skip
                lda doorl,y
                cmp level
                bne dsr_skip
                cpy #30
                beq dsr_keycard1
                cpy #40
                beq dsr_keycard2
                cpy #42
                beq dsr_keycard2
dsr_ok:         lda doord,y
                tay
                lda doorx,y
                sta warpdx
                lda doory,y
                clc
                adc #$01
                sta warpdy
                lda doorl,y
                sta warpdl
                lda #$01
                sta warp
                rts
dsr_skip:       dey
                bpl dsr_loop
dsr_quit:       rts
dsr_keycard2:   jmp dsr_kc2
dsr_keycard1:   stx alo
                sty ahi
                lda keycard1
                bne dsr_ok
                lda #ITEM_KEYCARD1
                jsr finditem
                bcs dsr_keyfound1
                lda #MSG_KEYCARD
                ldx #200
                jsr newmsg
                ldx alo
                ldy ahi
                rts
dsr_keyfound1:  inc keycard1
                jsr decammo2
                ldx alo
                ldy ahi
                jmp dsr_ok

dsr_kc2:        stx alo
                sty ahi
                lda keycard2
                bne dsr_ok
                lda #ITEM_KEYCARD2
                jsr finditem
                bcs dsr_keyfound2
                lda #MSG_KEYCARD
                ldx #200
                jsr newmsg
                ldx alo
                ldy ahi
                rts
dsr_keyfound2:  lda ahi                         ;Remove keycard 2 only when
                cmp #42                         ;entering the wasteland
                bne dsr_keyfound2b
                inc keycard2
                jsr decammo2
dsr_keyfound2b: ldx alo
                ldy ahi
                jmp dsr_ok


;Ŀ
;CHECKWARP                                                                    
;                                                                             
;Checks if player is supposed to warp into another location in the game world.
;                                                                             
;Parameters: warp=1 perform warp                                              
;            warpdx,dy,dl target x,y,level                                    
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

checkwarp:      lda warp
                bne chkw_noskip
                rts
chkw_noskip:    ldx #$05
                lda #$00
                sta shpxh
                sta shpyh
                lda warpdx
                sta shpxl
                lda warpdy
                sta shpyl
chkw_loop:      asl shpxl
                rol shpxh
                asl shpyl
                rol shpyh
                dex
                bne chkw_loop
                lda shpxl
                ora #$10
                sta shpxl
                jsr removeshapes
                jsr putleveldata
                lda showcut4
                bne chkw_nocut
                lda warpdl
                cmp #LVL_DESERT
                bne chkw_nocut
                lda warpdx
                cmp #5
                bne chkw_nocut
                lda warpdy
                cmp #11
                bne chkw_nocut
                inc showcut4
                lda #$04
                jsr cutscene
chkw_nocut:     lda warpdl
                jsr loadlevel
                jsr getleveldata
                lda warpdx              ;Take X-block position
                sec
                sbc #$05                ;Center player
                bcs inl_notover1
                sec
                lda #$00
inl_notover1:   tax
                lda warpdy              ;Do same for Y-coord
                sbc #$04
                bcs inl_notover2
                lda #$00
inl_notover2:   tay
                lda #SCRM_BLANK         ;Blank screen while loading
                sta screenmode
                sta warp
                jsr clearsprites                ;Sprites off
                jsr setmappos                   ;Set map position, draw screen
                jsr drawscreen
                jsr leveltune                   ;Init level music
                lda #SCRM_GAME                  ;Make screen visible
                sta screenmode
                jsr getcontrols                 ;Take new controls
                lda murder                      ;Special code for the
                beq inl_nomurder                ;assasination attempt
                lda level
                cmp #LVL_CITY                   ;(the enemies "follow" player
                bne inl_nomurder                ;until both killed)
                lda LEVELWORK+$3e+$80
                ora LEVELWORK+$3f+$80
                beq inl_murderend
                lda warpdx
                sta LEVELWORK+$3e
                sta LEVELWORK+$3f
                lda warpdy
                sta LEVELWORK+$3e+$40
                sta LEVELWORK+$3f+$40
                ldx #200
                lda #MSG_HITMEN
                jsr newmsg
inl_nomurder:   rts
inl_murderend:  lda #$00
                sta murder
                rts

        ;Background animation routine

bganim:         lda level
                cmp #$03
                beq bganim3
                cmp #$06
                beq bganim3
                cmp #$04
                beq bganim4
                cmp #$07
                bne bg_none
                jmp bganim7
bg_none:        rts
bganim4:        inc bgdelay2+1
bgdelay2:       lda #$00
                and #$03
                beq bganim4_1
                rts
bganim4_1:      ldx #$07
bganim4_2:      lda $c000+105*8,x
                sta alo
                lda $c000+106*8,x
                sta $c000+105*8,x
                lda $c000+107*8,x
                sta $c000+106*8,x
                lda $c000+108*8,x
                sta $c000+107*8,x
                lda alo
                sta $c000+108*8,x
                dex
                bpl bganim4_2
                rts
bganim3:        inc bgdelay+1
bgdelay:        lda #$00
                and #$01
                beq bganim3_2
bganim3_1:      lda $c000+102*8+14
                sta alo
                lda $c000+102*8+15
                sta ahi
                ldx #$0c
bganim3l:       lda $c000+102*8,x
                sta $c000+102*8+2,x
                lda $c000+102*8+1,x
                sta $c000+102*8+3,x
                dex
                dex
                bpl bganim3l
                lda alo
                sta $c000+102*8
                lda ahi
                sta $c000+102*8+1
                rts
bganim3_2:      ldx #$07
bganim3l2:      lda $c000+104*8,x
                sta alo
                lda $c000+105*8,x
                sta $c000+104*8,x
                lda $c000+106*8,x
                sta $c000+105*8,x
                lda $c000+107*8,x
                sta $c000+106*8,x
                lda alo
                sta $c000+107*8,x
                dex
                bpl bganim3l2
                rts
bganim7:        inc bgdelay7+1
bgdelay7:       lda #$00
                and #$03
                beq bganim7_1
                rts
bganim7_1:      ldx #$07
bganim7_2:      lda $c000+103*8,x
                sta alo
                lda $c000+104*8,x
                sta $c000+103*8,x
                lda $c000+105*8,x
                sta $c000+104*8,x
                lda $c000+106*8,x
                sta $c000+105*8,x
                lda alo
                sta $c000+106*8,x
                dex
                bpl bganim7_2
                rts


