;ͻ
;Raster interrupt routines, including spritemultiplexing                      
;ͼ

RASTER0LINE     = 189
RASTER1LINE     = 198
RASTER1BLINE    = 205
RASTER2LINE     = 16
RASTER4LINE     = 80

MINSPRY         = 30
MAXSPRY         = 187

;Ŀ
;INITRASTER                                                                   
;                                                                             
;Inits raster IRQs, turns Kernal off, inits some video registers.             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

initraster:     sei
                lda #$35                    ;All RAM except IO-area
                sta $01
                lda #<load_irq
                sta $0314
                lda #>load_irq
                sta $0315
                lda #<raster0
                sta $fffe
                lda #>raster0
                sta $ffff
                lda #<nmi                   ;Set NMI vector
                sta $0318
                sta $fffa
                lda #>nmi
                sta $0319
                sta $fffb
                lda #$7f                    ;Timer interrupt off
                sta $dc0d
                lda #$01                    ;Raster interrupt on
                sta $d01a
                lda #27                     ;Highbyte of interrupt position = 0
                sta $d011
                lda #RASTER0LINE
                sta $d012
                lda $dd00
                and #$fc
                sta $dd00                   ;Videobank at $c000:n
                lda #$00                    ;Black border
                sta $d020
                sta $d021
                sta $d015                   ;Sprites off
                sta $d01b                   ;Sprites on top of BG
                sta $d017                   ;Sprites not magnified
                sta $d01d
                lda #MUS_STOP
                sta playmusic+1             ;Stop music
                jsr resetsprites            ;Reset spritemultiplexing
                jsr clearsprites
                jsr initmsg                 ;Reset message system
                lda $dc0d                   ;Acknowledge IRQ for safety
                cli
                rts

load_irq:       lda fastloadstatus          ;In fakeIRQload-mode, redirect IRQs
                bmi redirectirq
                inc $d019
                jmp $ea81
                
redirectirq:    ldx $01
                lda #$35                        ;Note: this will necessarily have overhead,
                sta $01                         ;which means that the sensitive IRQs like
                lda #>rr_return                 ;the panel-split should take extra advance
                pha
                lda #<rr_return
                pha
                php
                jmp ($fffe)
rr_return:      stx $01
                jmp $ea81


;Ŀ
;WAITRASTER                                                                   
;                                                                             
;Waits for scorepanel                                                         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

waitraster:     ldx #$01
waitraster2:    stx waitcmp+1
                lda #$00
                sta rastercount
waitraster3:    lda rastercount
waitcmp:        cmp #$00
                bcc waitraster3
                rts

;Ŀ
;Different raster interrupts                                                  
;0 Scorescreen split. Sets $d011 and $d018.                                   
;1 Scorescreen visible part begins. Setup inventory sprites & play music      
;2 Gamescreen setup. Jump to spritemultiplexing to set first 8 sprites.       
;3 Spritemultiplexing. Loop until all sprites shown.                          
;4 Multicolormode change in the titlescreen                                   
;                                                                             
;Parameters: -                                                                
;Returns: X:number of next interrupt (IMPORTANT!)                             
;Modifies: A,X,Y                                                              
;

r0_direct:      lda #$57
                sta $d011
                bne r0_setsprites

raster0:        cld
                sta savea
                lda $d011
                cmp #$15
                beq r0_badline
r0_d011value:   lda #$17
                sta $d011
                pha
                pla
                pha
                pla
                nop
                nop
                nop
                nop
r0_badline:     lda #$5f                    ;Blank screen
                sta $d011
                stx savex
                sty savey
                inc $d019
r0_setsprites:  lda #$7f                    ;Set empty sprites
                sta SCREEN1+$3f8
                sta SCREEN1+$3f9
                sta SCREEN1+$3fa
                sta SCREEN1+$3fb
                sta SCREEN1+$3fc
                sta SCREEN1+$3fd
                sta SCREEN1+$3fe
                sta SCREEN1+$3ff
                lda #$80                    ;Scorescreen screen memory pointer
                sta $d018
                lda #$00                    ;Black background
                sta $d021
                inc rastercount
                lda fileopen                ;If file open, execute raster1 directly
                bne raster1_direct          ;because of possible glitches
                lda #RASTER1LINE
                sta $d012
                lda #<raster1
                sta $fffe
                lda #>raster1
                sta $ffff
                lda savea
                ldx savex
                ldy savey
nmi:            rti

raster1_direct: lda $d012
                cmp #RASTER1LINE
                bcc raster1_direct
                pha
                pla
                pha
                pla
                nop
                nop
                bcs raster1_direct2

raster1:        cld
                sta savea
                stx savex
                sty savey
                inc $d019
raster1_direct2:lda #$18                    ;Immobilize X-scrolling
                sta $d016
                lda #$07                    ;Set scorescreen X-colors
                sta $d022
                lda #$08
                sta $d023
                lda #$00
                sta $d010
                lda #$17                    ;Turn on screen
                sta $d011
                lda #$0b
                sta $d026
                lda #$ff                    ;All sprites on and multicolor
                sta $d015
                sta $d01c
                lda #$0f                    ;Sprite multicolor 1
                sta $d025
N               SET 0                       ;Set inventory sprites X & Y
                REPEAT 8
                lda #$1d+N*24
                sta $d000+N*2
N               SET N+1
                REPEND
                lda #210
N               SET 0
                REPEAT 8
                sta $d001+N*2
N               SET N+1
                REPEND
                lda #1
N               SET 0
                REPEAT 8
                sta $d027+N                 ;Set inventory colors
N               SET N+1
                REPEND
                lda #RASTER1BLINE
                sta $d012
                lda #<raster1b
                sta $fffe
                lda #>raster1b
                sta $ffff
                lda savea
                ldx savex
                ldy savey
                rti

raster1b:       cld
                sta savea
                stx savex
                sty savey
                inc $d019
                ldx #$00
r1_invfloop:    lda plrinvt,x               ;Set inventory frames
                bmi r1_invskip
                tay
                lda itemframetbl,y
r1_invempty:    sta SCREEN1+1016,x
r1_invskip:     inx
                cpx #$08
                bcc r1_invfloop
r1_invdone:     jsr dosfx                   ;Play music/sounds
                lda #RASTER2LINE
                sta $d012
                lda #<raster2
                sta $fffe
                lda #>raster2
                sta $ffff
                lda savea
                ldx savex
                ldy savey
                rti

raster2:        cld
                sta savea
                stx savex
                sty savey
                inc $d019
r2_checksprites:lda newsprites              ;New sprites for display?
                beq r2_setx
                dec newsprites
                lda sprirqfirst
                sta irqfirst
                lda sprirqlast
                sta irqlast
                lda sprlast
                sec
                sbc sprfirst
                sta r2_numspr+1
r2_setx:        ldx #$14                    ;Scorescreen scrolling
r2_sety:        lda #$14
                ldy screenmode              ;Check screen mode
                cpy #SCRM_TITLE
                beq r2_title                ;TITLE?
                cpy #SCRM_GAME              ;GAME?
                beq r2_game
r2_blank:       ora #$47                    ;BLANK?
r2_title:       ldx #$18
r2_game:        sta $d011
                ora #$07
                sta r0_d011value+1
                stx $d016
                lda bossdeath
                cmp #$0a
                bcc r2_setbgok
                lda #$00
r2_setbgok:     lsr
                tax
                lda setbg,x
                sta $d021
                lda setmc1,x
                sta $d022
                lda setmc2,x
                sta $d023
r2_setmulticol: lda #$ff                    ;Set multicolor sprites
                sta $d01c
r2_setsprmc1:   lda #SPRMC1                 ;Sprite multicolor 1
                sta $d025
r2_setsprmc2:   lda #SPRMC2
                sta $d026                   ;Sprite multicolor 2
                ldx screen                  ;Scorescreen screen memory pointer
                lda scrd018,x
                sta $d018
                jsr updmsg                  ;Update message screen
                lda screenmode              ;If screen is blank, no sprites at
                cmp #SCRM_GAME              ;all
                beq r2_noblank2
                lda #$00
                sta $d015
                lda screenmode              ;Extra interrupt in titlescreen
                cmp #SCRM_TITLE
                bcc r2_setraster0
r2_setraster4:  lda #RASTER4LINE
                sta $d012
                lda #<raster4
                sta $fffe
                lda #>raster4
                sta $ffff
                lda savea
                ldx savex
                ldy savey
                rti
r2_noblank2:
r2_numspr:      ldy #$00
                cpy #$09
                bcc r2_notover1
                ldy #$08
r2_notover1:    lda ontable,y
                sta $d015
                ldx irqfirst                    ;Any sprite-IRQs?
                cpx irqlast
                bne r2_sprirq                   ;If not, go back
r2_setraster0:  lda #RASTER0LINE
                sta $d012
                lda #<raster0
                sta $fffe
                lda #>raster0
                sta $ffff
                lda savea
                ldx savex
                ldy savey
                rti
r2_sprirq:      stx irqcount
                ldy screen
                lda scrframetbl,y               ;Load right frames in spriteIRQ
                sta r3_frame+2
                lda sprirqstart,x
                sta r3_direct+1
                sta r3_reload+1
                lda sprirqend,x
                sta r3_cmp1+1
                sta r3_cmp2+1
                lda #<raster3
                sta $fffe
                lda #>raster3
                sta $ffff
                jmp r3_direct

raster3:        cld
                sta savea
                stx savex
                sty savey
                inc $d019
r3_direct:      ldy #$00
r3_loop1:       ldx physsprtbl2,y
                lda sortspry,y
                sta $d001,x
                iny
r3_cmp1:        cpy #$00
                bne r3_loop1
r3_reload:      ldy #$00
r3_loop2:       ldx physsprtbl,y
                lda sortsprf,y
r3_frame:       sta SCREEN1+$3f8,x
                lda sortsprc,y
                sta $d027,x
                ldx physsprtbl2,y
                lda sortsprx,y
                sta $d000,x
                lda sortsprxh,y
                sta $d010
                iny
r3_cmp2:        cpy #$00
                bne r3_loop2
                inc irqcount
                ldx irqcount
                cpx irqlast                 ;SpriteIRQs remaining?
                beq r3_setraster0
r3_notlast:     lda sprirqstart,x           ;Take first spritenumber
                sta r3_direct+1
                sta r3_reload+1
                lda sprirqend,x
                sta r3_cmp1+1
                sta r3_cmp2+1
                sec                         ;Calculate the "advance" in Y-
                sbc sprirqstart,x           ;position for the next spriteIRQ
                tay
                lda sprirqpostbl-1,y
                sta r3_sub+1
                lda sprirqline,x
r3_sub:         sbc #$00
                cmp $d012                   ;Late from interrupt?
                beq r3_late
                bcs r3_notlate
r3_late:        jmp r3_direct
r3_notlate:     adc #$00                    ;adc #1, carry is set
                sta $d012
                lda savea
                ldx savex
                ldy savey
                rti
r3_setraster0:  lda #RASTER0LINE-2          ;Late from interrupt 0?
                cmp $d012
                bcs r3_notlate2
                jmp r0_direct
r3_notlate2:    jmp r2_setraster0

raster4:        cld
                sta savea
                stx savex
                sty savey
                inc $d019
                lda #$08                    ;Multicolormode off
                sta $d016
                jmp r2_setraster0

;ͻ
;Spriteroutines, >8 sprites onscreen                                          
;ͼ

;Ŀ
;RESETSPRITES                                                                 
;                                                                             
;Resets sprite multiplex system & builds initial sort-order.                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

resetsprites:   lda #$00
                sta sprirqfirst
                sta sprfirst
                sta irqfirst
                ldx #MAX_SPR
rs_sortorder:   txa
                sta sortorder,x
                dex
                bpl rs_sortorder
                rts

;Ŀ
;CLEARSPRITES                                                                 
;                                                                             
;Removes sprites from screen and resets multiplexing.                         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

clearsprites:   lda irqfirst
                sta irqlast
                lda #$00
                sta r2_numspr+1
                sta newsprites
                rts

;Ŀ
;GETSPRCOLS                                                                   
;                                                                             
;Updates spritecolors 64-254. (Using the unused 63rd byte in spritedatas)     
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

getsprcols:     sei
                lda #$34
                sta $01
                ldx #$40
                lda #$ff
                sta gccomp+1
                lda #$d0
                sta scrolltemp2
                jsr gc
                lda #$35
                sta $01
                cli
                lda #$01                ;Color 255 is white for flashing the
                sta SPRCOLS+255         ;sprites
                rts

;Ŀ
;GETBOSSCOLS                                                                  
;                                                                             
;Updates sprite colors 32-64 (level specific sprites)                         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

getbosscols:    ldx #$20
                lda #$40
                sta gccomp+1
                lda #$c8
                sta scrolltemp2
                jmp gc

gc:             lda #$3f
                sta scrolltemp1
                ldy #$00
gcloop:         lda (scrolltemp1),y
                sta SPRCOLS,x
                lda scrolltemp1
                clc
                adc #$40
                sta scrolltemp1
                bcc gcnotover
                inc scrolltemp2
gcnotover:      inx
gccomp:         cpx #$00
                bne gcloop
                rts

;Ŀ
;UPDATESPRITES                                                                
;                                                                             
;Sorts sprites to top-bottom order                                            
;To be called after UPDATESHAPES                                              
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

updatesprites:  lda newsprites            ;Wait for previous update
                bne updatesprites
                ldx numspr
                lda #$ff                  ;Set the Y-coords of the
upds_unusedsprloop:                       ;unused sprites to $ff
                sta spry,x
                inx
                cpx #MAX_SPR+1
                bcc upds_unusedsprloop
                lda sprirqfirst           ;Switch doublebuffer half
                eor #MAX_SPRIRQ
                sta sprirqfirst
                sta sprirqlast            ;Assume: no sprites
                lda sprfirst
                eor #MAX_SPR
                sta sprfirst
                clc
                adc #$08
                sta upds_copyycmp+1
                ldx #$00
upds_sortloop:  ldy sortorder+1,x
                lda spry,y
                ldy sortorder,x
                cmp spry,y
                bcs upds_sortskip
                stx upds_sortreload+1
upds_sortswap:  lda sortorder+1,x
                sta sortorder,x
                sty sortorder+1,x
                cpx #$00
                beq upds_sortreload
                dex
                ldy sortorder+1,x
                lda spry,y
                ldy sortorder,x
                cmp spry,y
                bcc upds_sortswap
upds_sortreload:ldx #$00
upds_sortskip:  inx
                cpx #MAX_SPR-1
                bcc upds_sortloop
                lda #<sortorder
                sta upds_copyloop+1
                ldy sprfirst
upds_copyloop:  ldx sortorder
                inc upds_copyloop+1
                lda spry,x
                cmp #MAXSPRY
                bcs upds_copyready
                cmp #MINSPRY
                bcc upds_copyloop
                sta sortspry,y
upds_copyycmp:  cpy #$00
                bcc upds_copyycmpskip
                sbc #21
                cmp sortspry-8,y                ;Check for too many sprites on
                bcc upds_copyloop               ;same row
upds_copyycmpskip:
                lda sprx,x
                sta sortsprx,y
                lda sprxh,x
                sta sortsprxh,y
                lda sprf,x
                sta sortsprf,y
                ora sprfls,x
                sta upds_getcolor+1
upds_getcolor:  lda SPRCOLS
                sta sortsprc,y
                iny
                bne upds_copyloop
upds_copyready: sty sprlast
                cpy sprfirst
                bne upds_notzero
                rts
upds_notzero:   ldx sprfirst                ;Logical spritenumber
                ldy sprirqfirst             ;SpriteIRQ-number
                tya
                clc
                adc #MAX_SPRIRQ
                sta upds_irqc+1             ;Last IRQ-number that can be used
                lda #$00
                sta tempd010
upds_irqloop:   lda sortspry,x
                sta sprirqline,y            ;Set IRQ position
upds_ok:        txa
                sta sprirqstart,y
                clc
                adc #$08
                sta upds_c+1
upds_irqsprloop:lda sortsprxh,x             ;Handle sprite X-coord MSB
                lsr
                lda tempd010
                bcc upds_notover255
                ora ortable,x
                bne upds_xhighdone
upds_notover255:and andtable,x
upds_xhighdone: sta tempd010
                sta sortsprxh,x
                inx
                cpx sprlast                 ;Out of logical sprites?
                beq upds_lastirq
upds_c:         cpx #$00                    ;SpriteIRQ full? (max.8 sprites at
                beq upds_nextirq            ;once)
                cpy sprirqfirst             ;Overwrite check not needed for first IRQ
                beq upds_irqsprloop
                lda sortspry-8,x            ;Can next sprite be overwritten?
                clc
                adc #19
                cmp sprirqline,y
                bcc upds_irqsprloop
upds_nextirq:   txa
                sta sprirqend,y
                iny
upds_irqc:      cpy #$00
                beq upds_done
                jmp upds_irqloop
upds_lastirq:   txa
                sta sprirqend,y
                iny
upds_done:      sty sprirqlast
                rts

;Ŀ
;INITMSG                                                                      
;                                                                             
;Initializes message system.                                                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

initmsg:        lda #$ff
                sta msgnum
                lda #$00
                sta msgcnt
                ldx #40
                lda #$20
imsg_loop:      sta SCREEN1+18*40-1,x
                dex
                bne imsg_loop
                rts

;Ŀ
;NEWMSG                                                                       
;                                                                             
;Gives new message to be displayed to the message system                      
;                                                                             
;Parameters: A:message number                                                 
;            X:time onscreen                                                  
;Returns: -                                                                   
;Modifies: -                                                                  
;

newmsg:         stx msgtime
                sta msgnum
                rts

;Ŀ
;UPDMSG                                                                       
;                                                                             
;Message update (scrolling) routine. Called from a raster interrupt.          
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

updmsg:         lda msgnum
                cmp #$fe
                beq um_wait
                bcc um_new
                lda msgcnt
                beq um_done
                dec msgcnt
                jsr um_scroll
                lda #32
                sta SCREEN1+18*40+39
um_done:        rts
um_new:         asl
                tax
                lda msgptr,x
                sta msglo
                lda msgptr+1,x
                sta msghi
                lda #40
                sta msgcnt
                lda #$fe
                sta msgnum
                rts
um_wait:        lda msgcnt
                beq um_wait2
                dec msgcnt
                jsr um_scroll
                ldy #$00
                lda (msglo),y
                bne um_notend
                lda #$20
                sta SCREEN1+18*40+39
                rts
um_notend:      cmp #$40
                bcc um_printok
                sbc #$40
um_printok:     sta SCREEN1+18*40+39
                inc msglo
                bne um_notover
                inc msghi
um_notover:     rts
um_wait2:       lda msgtime
                cmp #$ff
                sbc #$00
                sta msgtime
                bne um_notover2
                lda #$ff
                sta msgnum
                lda #40
                sta msgcnt
um_notover2:    rts
um_scroll:      ldx #$00
um_scrollloop:  lda SCREEN1+18*40+1,x
                sta SCREEN1+18*40,x
                inx
                cpx #39
                bcc um_scrollloop
                rts


