;ͻ
;Scroll- and background graphics routines                                     
;ͼ

;Ŀ
;INITMAP                                                                      
;                                                                             
;Calculates start addresses for each row of the blockmap.                     
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

initmap:        lda #<MAP
                sta scrolltemp1
                lda #>MAP
                sta scrolltemp2
                ldx #$00
im_loop:        lda scrolltemp1
                sta maptbllo,x
                lda scrolltemp2
                sta maptblhi,x
                lda scrolltemp1
                clc
                adc mapsx
                sta scrolltemp1
                bcc im_notover
                inc scrolltemp2
im_notover:     inx
                cpx #$80
                bne im_loop
                rts

;Ŀ
;INITBLOCKS                                                                   
;                                                                             
;Calculates address for each background graphics block.                       
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

initblocks:     lda #<BLOCKS
                sta scrolltemp1
                lda #>BLOCKS
                sta scrolltemp2
                ldx #$00
ib_loop:        lda scrolltemp1
                sta blktbllo,x
                lda scrolltemp2
                sta blktblhi,x
                lda scrolltemp1
                clc
                adc #$10
                sta scrolltemp1
                bcc ib_notover
                inc scrolltemp2
ib_notover:     inx
                cpx #$80
                bne ib_loop
                rts

;Ŀ
;INITSCROLL                                                                   
;                                                                             
;Resets scrolling zeropage variables and sets finescroll so that whole chars  
;are visible. (when screen is actually being scrolled, finescroll is centered 
;whenever scrolling has been stopped)                                         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

initscroll:     jsr is_clear
                sta screen
                lda #$07
is_setscrollpos:sta scrollx
                sta scrolly
                ora #$10
                sta r2_setx+1
                sta r2_sety+1
                rts
is_clear:       lda #$00
                sta scrollsx
                sta scrollsy
                sta scrollcsx
                sta scrollcsy
                sta scrcounter
                sta scradd
                rts

;Ŀ
;SETMAPPOS                                                                    
;                                                                             
;Sets map position, and centers the finescroll registers.                     
;                                                                             
;Parameters: X,Y - X and Y position in blocks                                 
;Returns: -                                                                   
;Modifies: A                                                                  
;

setmappos:      stx mapx
                sty mapy
                lda #$02
                sta blockx
                lda #$01
                sta blocky
                lda #$04
                jsr is_setscrollpos
                jmp ds_setsprpos

;Ŀ
;DOSCROLL                                                                     
;                                                                             
;Performs scrolling logic.                                                    
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

doscroll:       lda scradd                      ;If speed 0, wait for new
                beq ds_getnewspeed              ;(nonzero) speed request
                lda scrollx                     ;Update finescrolling
                sec
                sbc scrollcsx                   ;(don't care of overflow yet)
                and #$07
                sta scrollx
                lda scrolly
                sec
                sbc scrollcsy
                and #$07
                sta scrolly
                lda scrcounter                  ;Then increase scrollcounter
                clc                             ;(amount of scrolled pixels)
                adc scradd
                sta scrcounter
                cmp #$08                        ;If it's >7, the scrolling has
                bcs ds_getnewspeed              ;completed
                cmp #$04                        ;If 4, prepare to swap screens
                beq ds_prepare                  ;and update graphics in the
                jmp ds_setsprpos                ;border,
ds_prepare:     lda scrollcsx                   ;as well as change the block-
                beq ds_pxdone                   ;and mapposition variables
                bmi ds_goleft
ds_goright:     inc blockx
                lda blockx
                cmp #4
                bcc ds_pxdone
                lda #0
                sta blockx
                inc mapx
                jmp ds_pxdone
ds_goleft:      dec blockx
                bpl ds_pxdone
                lda #3
                sta blockx
                dec mapx
ds_pxdone:      lda scrollcsy
                beq ds_pydone
                bmi ds_goup
ds_godown:      inc blocky
                lda blocky
                cmp #4
                bcc ds_pydone
                lda #0
                sta blocky
                inc mapy
                jmp ds_pydone
ds_goup:        dec blocky
                bpl ds_pydone
                lda #3
                sta blocky
                dec mapy
ds_pydone:      jmp ds_setsprpos

ds_getnewspeed: lda #$00
                sta scrcounter
                sta scradd                      ;Assume zero speed
                lda scrollsy
                sta scrollcsy
                lda scrollsx                    ;Copy speed request into the
                sta scrollcsx                   ;actual scrolling speed
                bpl ds_xnotneg                  ;Take absolute value of X-speed
                eor #$ff
                clc
                adc #$01
ds_xnotneg:     sta scradd                      ;And store as the scrolling
                sta scrolltemp1                 ;counter-addvalue
                lda scrollcsy
                bpl ds_ynotneg
                eor #$ff
                clc
                adc #$01
ds_ynotneg:     sta scrolltemp2
                cmp scradd                      ;Is Y-absolute greater?
                bcc ds_ynothigher               ;If it is, use it as the
                sta scradd                      ;addvalue instead
ds_ynothigher:  lda scradd                      ;Scrolling at all?
                bne ds_not0
ds_setsprpos:   ldx #0                          ;Calculate the subtract-value
                lda mapx                        ;for drawing sprites
ds_sspxnotneg:  stx scrposxh
                asl
                rol scrposxh
                asl
                rol scrposxh
                asl
                rol scrposxh
                asl
                rol scrposxh
                asl
                rol scrposxh
                sta scrposxl
                lda blockx
                asl
                asl
                asl
                adc scrposxl
                sta scrposxl
                bcc ds_ssp1
                inc scrposxh
ds_ssp1:        lda #24
                clc
                adc scrollx
                sta scrolltemp1
                lda scrposxl
                sec
                sbc scrolltemp1
                sta scrposxl
                bcs ds_ssp2
                dec scrposxh
ds_ssp2:        ldx #0
ds_sspynotneg:  stx scrposyh
                lda mapy
                asl
                rol scrposyh
                asl
                rol scrposyh
                asl
                rol scrposyh
                asl
                rol scrposyh
                asl
                rol scrposyh
                sta scrposyl
                lda blocky
                asl
                asl
                asl
                adc scrposyl
                sta scrposyl
                bcc ds_ssp3
                inc scrposyh
ds_ssp3:        lda #47
                clc
                adc scrolly
                sta scrolltemp1
                lda scrposyl
                sec
                sbc scrolltemp1
                sta scrposyl
                bcs ds_ssp4
                dec scrposyh
ds_ssp4:        rts

ds_not0:        cmp #$02                        ;Maximum speed 2?
                bne ds_not2
                lda scrolltemp1                 ;If the speed in the
                cmp #$01                        ;"other" axis is only 1,
                bne ds_xnot1                    ;it must be increased
                asl scrollcsx
ds_xnot1:       lda scrolltemp2
                cmp #$01
                bne ds_ynot1
                asl scrollcsy
ds_not2:
ds_ynot1:       ldx screen                      ;Which screen
                lda shiftjumptbllo,x            ;Take the corresponding screen
                sta ds_shiftjump+1              ;shift routine
                lda shiftjumptblhi,x
                sta ds_shiftjump+2
                ldy #40                         ;Y=source index
                lda scrollcsx                   ;Modify it according to the
                beq ds_nox                      ;scrolling direction
                bmi ds_left
ds_right:       lda #3
                sta scrollx
                iny
                jmp ds_nox
ds_left:        lda #4
                sta scrollx
                dey
ds_nox:         lda scrollcsy
                beq ds_noy
                bmi ds_up
ds_down:        lda #3
                sta scrolly
                tya
                clc
                adc #40
                tay
                jmp ds_noy
ds_up:          lda #4
                sta scrolly
                tya
                sec
                sbc #40
                tay
ds_noy:         sty shiftpos
                jmp ds_setsprpos

;Ŀ
;SHIFTSCREEN                                                                  
;                                                                             
;Performs "hard work" of scrolling. To be called during scorescreen!          
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

shiftscreen:    lda scrollx
                ora #$10
                sta r2_setx+1
                lda scrolly
                ora #$10
                sta r2_sety+1
                lda scradd                      ;If speed 0, do nothing
                beq ss_done
                lda scrcounter
                beq ss_firsthalf                ;First or second half of
                cmp #$02                        ;screen shifting?
                beq ss_secondhalf
                cmp #$04                        ;Or colorscreen scrolling?
                beq ss_colors
ss_done:        rts

ss_firsthalf:   ldx #0                          ;First half now...
                lda #20
ss_fh1:         sta scrolltemp1
                ldy shiftpos
ds_shiftjump:   jmp ds_shift1

ss_secondhalf:  ldx #20
                lda #39
                jmp ss_fh1

ss_colors:      lda screen                      ;Swap screens now
                eor #$01
                sta screen
                lda scrollcsx
                beq ds_cnox
                bmi ds_cleft
ds_cright:      jsr ds_shiftcolright            ;Scroll the color-RAM and
                jsr ds_drawright                ;draw new graphics to the
                jmp ds_cnox                     ;borders
ds_cleft:       jsr ds_shiftcolleft
                jsr ds_drawleft
ds_cnox:        lda scrollcsy
                beq ds_cnoy
                bmi ds_cup
ds_cdown:       jsr ds_shiftcoldown
                jmp ds_drawdown
ds_cup:         jsr ds_shiftcolup
                jmp ds_drawup
ds_cnoy:        rts



ds_shift1:
ds_shiftloop1:
N               SET 0
                REPEAT ROWS
                lda SCREEN1-40+N*40,y
                sta SCREEN2+N*40,x
N               SET N+1
                REPEND
                iny
                inx
                cpx scrolltemp1
                bne ds_shiftloop1
                sty shiftpos
                rts

ds_shift2:      ldy shiftpos
                sta scrolltemp1
ds_shiftloop2:
N               SET 0
                REPEAT ROWS
                lda SCREEN2-40+N*40,y
                sta SCREEN1+N*40,x
N               SET N+1
                REPEND
                iny
                inx
                cpx scrolltemp1
                bne ds_shiftloop2
                sty shiftpos
                rts

ds_drawleft:    ldx mapy
                lda maptbllo,x
                sta maptemplo
                lda maptblhi,x
                sta maptemphi
                lda mapx
                clc
                adc maptemplo
                sta maptemplo
                bcc ds_dlcalcdone
                inc maptemphi
ds_dlcalcdone:  ldx screen
                lda #$00
                sta dsdl_sta+1
                lda scradrtblhi,x
                sta dsdl_sta+2
                lda #<COLORS
                sta dsdl_sta2+1
                lda #>COLORS
                sta dsdl_sta2+2
                lda blocky
                asl
                asl
                sta scrolltemp2
                ldx #ROWS-1
                ldy #0
                lda (maptemplo),y
                tay
                lda BLOCKCOLS,y
                sta dsdl_lda2+1
                lda blktbllo,y
                clc
                adc blockx
                sta dsdl_lda+1
                lda blktblhi,y
                sta dsdl_lda+2
                ldy scrolltemp2
dsdl_lda:       lda $1000,y
dsdl_sta:       sta $1000
dsdl_lda2:      lda #$00
dsdl_sta2:      sta $1000
                dex
                bmi dsdl_ready
                lda dsdl_sta+1
                clc
                adc #40
                sta dsdl_sta+1
                sta dsdl_sta2+1
                bcc dsdl_not2
                inc dsdl_sta+2
                inc dsdl_sta2+2
                clc
dsdl_not2:      tya
                adc #4
                cmp #16
                beq dsdl_getblock
                tay
                jmp dsdl_lda
dsdl_getblock:  lda maptemplo
                clc
                adc mapsx
                sta maptemplo
                bcc dsdl_not3
                inc maptemphi
dsdl_not3:      ldy #0
                lda (maptemplo),y
                tay
                lda BLOCKCOLS,y
                sta dsdl_lda2+1
                lda blktbllo,y
                clc
                adc blockx
                sta dsdl_lda+1
                lda blktblhi,y
                sta dsdl_lda+2
                ldy #0
                jmp dsdl_lda
dsdl_ready:     rts

ds_drawright:   ldx mapy
                lda maptbllo,x
                sta maptemplo
                lda maptblhi,x
                sta maptemphi
                lda mapx
                clc
                adc #9
                clc
                adc maptemplo
                sta maptemplo
                bcc ds_drcalcdone
                inc maptemphi
ds_drcalcdone:  lda blockx
                clc
                adc #2
                sta scrolltemp1
                cmp #4
                bcc dsdr_not1
                and #3
                sta scrolltemp1
                lda maptemplo
                clc
                adc #1
                sta maptemplo
                bcc dsdr_not1
                inc maptemphi
dsdr_not1:      ldx screen
                lda #38
                sta dsdr_sta+1
                lda scradrtblhi,x
                sta dsdr_sta+2
                lda #<(COLORS+38)
                sta dsdr_sta2+1
                lda #>(COLORS+38)
                sta dsdr_sta2+2
                lda blocky
                asl
                asl
                sta scrolltemp2
                ldx #ROWS-1
                ldy #0
                lda (maptemplo),y
                tay
                lda BLOCKCOLS,y
                sta dsdr_lda2+1
                lda blktbllo,y
                clc
                adc scrolltemp1
                sta dsdr_lda+1
                lda blktblhi,y
                sta dsdr_lda+2
                ldy scrolltemp2
dsdr_lda:       lda $1000,y
dsdr_sta:       sta $1000
dsdr_lda2:      lda #$00
dsdr_sta2:      sta $1000
                dex
                bmi dsdr_ready
                lda dsdr_sta+1
                clc
                adc #40
                sta dsdr_sta+1
                sta dsdr_sta2+1
                bcc dsdr_not2
                inc dsdr_sta+2
                inc dsdr_sta2+2
                clc
dsdr_not2:      tya
                adc #4
                cmp #16
                beq dsdr_getblock
                tay
                jmp dsdr_lda
dsdr_getblock:  lda maptemplo
                clc
                adc mapsx
                sta maptemplo
                bcc dsdr_not3
                inc maptemphi
dsdr_not3:      ldy #0
                lda (maptemplo),y
                tay
                lda BLOCKCOLS,y
                sta dsdr_lda2+1
                lda blktbllo,y
                clc
                adc scrolltemp1
                sta dsdr_lda+1
                lda blktblhi,y
                sta dsdr_lda+2
                ldy #0
                jmp dsdr_lda
dsdr_ready:     rts

ds_drawup:      ldx mapy
                lda maptbllo,x
                sta maptemplo
                lda maptblhi,x
                sta maptemphi
                ldy mapx
                lda blocky
                asl
                asl
                sta scrolltemp1
                ldx screen
                lda scradrtblhi,x
                sta dsdu_sta+2
                ldx #0
                lda (maptemplo),y
                iny
                sty mapindex
                tay
                lda BLOCKCOLS,y
                sta dsdu_lda2+1
                lda blktbllo,y
                clc
                adc scrolltemp1
                sta dsdu_lda+1
                lda blktblhi,y
                sta dsdu_lda+2
                ldy blockx
dsdu_lda:       lda $1000,y
dsdu_sta:       sta SCREEN1,x
dsdu_lda2:      lda #$00
                sta COLORS,x
                inx
                cpx #39
                beq dsdu_ready
                iny
                cpy #4
                bne dsdu_lda
dsdu_getblock:  ldy mapindex
                inc mapindex
                lda (maptemplo),y
                tay
                lda BLOCKCOLS,y
                sta dsdu_lda2+1
                lda blktbllo,y
                clc
                adc scrolltemp1
                sta dsdu_lda+1
                lda blktblhi,y
                sta dsdu_lda+2
                ldy #0
                jmp dsdu_lda
dsdu_ready:     rts

ds_drawdown:    lda mapy
                clc
                adc #4
                tax
                lda blocky
                clc
                adc #1
                cmp #4
                bcc dsdd_notover
                sec
                sbc #4
                inx
dsdd_notover:   asl
                asl
                sta scrolltemp1
                lda maptbllo,x
                sta maptemplo
                lda maptblhi,x
                sta maptemphi
                ldy mapx
                ldx screen
                lda scradrtblhi,x
                clc
                adc #$02
                sta dsdd_sta+2
                ldx #0
                lda (maptemplo),y
                iny
                sty mapindex
                tay
                lda BLOCKCOLS,y
                sta dsdd_lda2+1
                lda blktbllo,y
                clc
                adc scrolltemp1
                sta dsdd_lda+1
                lda blktblhi,y
                sta dsdd_lda+2
                ldy blockx
dsdd_lda:       lda $1000,y
dsdd_sta:       sta SCREEN1+ROWS*40-40,x
dsdd_lda2:      lda #$00
                sta COLORS+ROWS*40-40,x
                inx
                cpx #39
                beq dsdd_ready
                iny
                cpy #4
                bne dsdd_lda
dsdd_getblock:  ldy mapindex
                inc mapindex
                lda (maptemplo),y
                tay
                lda BLOCKCOLS,y
                sta dsdd_lda2+1
                lda blktbllo,y
                clc
                adc scrolltemp1
                sta dsdd_lda+1
                lda blktblhi,y
                sta dsdd_lda+2
                ldy #0
                jmp dsdd_lda
dsdd_ready:     rts

ds_shiftcolleft:lda #2
                sec
                sbc blockx
                and #3
                tax
                clc
                adc #1
                tay
shclloop:
N               SET 0
                REPEAT ROWS
                lda COLORS+N*40,x
                sta COLORS+N*40,y
N               SET N+1
                REPEND
                txa
                clc
                adc #4
                tax
                adc #1
                tay
                cpy #39
                bcc shclloop
                rts

ds_shiftcolright:
                lda #4
                sec
                sbc blockx
                and #3
                tax
                clc
                adc #1
                tay
shcrloop:
N               SET 0
                REPEAT ROWS
                lda COLORS+N*40,y
                sta COLORS+N*40,x
N               SET N+1
                REPEND
                txa
                clc
                adc #4
                tax
                adc #1
                tay
                cpy #39
                bcc shcrloop
                rts

ds_shiftcolup:  lda blocky
                asl
                tax
                lda shcujumptbl,x
                sta shcujump+1
                lda shcujumptbl+1,x
                sta shcujump+2
                ldx #38
shcujump:       jmp $0000
shcujumptbl:    .word shcu0
                .word shcu1
                .word shcu2
                .word shcu3
shcu0:          lda COLORS+2*40,x
                sta COLORS+3*40,x
                lda COLORS+6*40,x
                sta COLORS+7*40,x
                lda COLORS+10*40,x
                sta COLORS+11*40,x
                lda COLORS+14*40,x
                sta COLORS+15*40,x
                dex
                bpl shcu0
                rts
shcu1:          lda COLORS+1*40,x
                sta COLORS+2*40,x
                lda COLORS+5*40,x
                sta COLORS+6*40,x
                lda COLORS+9*40,x
                sta COLORS+10*40,x
                lda COLORS+13*40,x
                sta COLORS+14*40,x
                ;lda COLORS+17*40,x
                ;sta COLORS+18*40,x
                dex
                bpl shcu1
                rts
shcu2:          lda COLORS+0*40,x
                sta COLORS+1*40,x
                lda COLORS+4*40,x
                sta COLORS+5*40,x
                lda COLORS+8*40,x
                sta COLORS+9*40,x
                lda COLORS+12*40,x
                sta COLORS+13*40,x
                lda COLORS+16*40,x
                sta COLORS+17*40,x
                dex
                bpl shcu2
                rts
shcu3:          lda COLORS+3*40,x
                sta COLORS+4*40,x
                lda COLORS+7*40,x
                sta COLORS+8*40,x
                lda COLORS+11*40,x
                sta COLORS+12*40,x
                lda COLORS+15*40,x
                sta COLORS+16*40,x
                dex
                bpl shcu3
                rts

ds_shiftcoldown:lda blocky
                asl
                tax
                lda shcdjumptbl,x
                sta shcdjump+1
                lda shcdjumptbl+1,x
                sta shcdjump+2
                ldx #38
shcdjump:       jmp $0000
shcdjumptbl:    .word shcd0
                .word shcd1
                .word shcd2
                .word shcd3
shcd0:          lda COLORS+1*40,x
                sta COLORS+0*40,x
                lda COLORS+5*40,x
                sta COLORS+4*40,x
                lda COLORS+9*40,x
                sta COLORS+8*40,x
                lda COLORS+13*40,x
                sta COLORS+12*40,x
                lda COLORS+17*40,x
                sta COLORS+16*40,x
                dex
                bpl shcd0
                rts
shcd1:          lda COLORS+4*40,x
                sta COLORS+3*40,x
                lda COLORS+8*40,x
                sta COLORS+7*40,x
                lda COLORS+12*40,x
                sta COLORS+11*40,x
                lda COLORS+16*40,x
                sta COLORS+15*40,x
                dex
                bpl shcd1
                rts
shcd2:          lda COLORS+3*40,x
                sta COLORS+2*40,x
                lda COLORS+7*40,x
                sta COLORS+6*40,x
                lda COLORS+11*40,x
                sta COLORS+10*40,x
                lda COLORS+15*40,x
                sta COLORS+14*40,x
                dex
                bpl shcd2
                rts
shcd3:          lda COLORS+2*40,x
                sta COLORS+1*40,x
                lda COLORS+6*40,x
                sta COLORS+5*40,x
                lda COLORS+10*40,x
                sta COLORS+9*40,x
                lda COLORS+14*40,x
                sta COLORS+13*40,x
                dex
                bpl shcd3
                rts

;Ŀ
;DRAWSCREEN                                                                   
;                                                                             
;Draws the whole screen. Screen should be blanked during this...              
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

drawscreen:     ldx mapy
                lda maptbllo,x
                sta maptemplo
                lda maptblhi,x
                sta maptemphi
                lda maptemplo
                clc
                adc mapx
                sta maptemplo
                bcc drsc_calcdone
                inc maptemphi
drsc_calcdone:  lda #ROWS
                sta scrolltemp2
                lda blocky
                asl
                asl
                sta scrolltemp1
                ldx screen
                lda #$00
                sta drsc_sta+1
                lda scradrtblhi,x
                sta drsc_sta+2
                lda #<COLORS
                sta drsc_sta2+1
                lda #>COLORS
                sta drsc_sta2+2
drsc_rowloop:   ldx #0
                ldy #0
                lda (maptemplo),y
                iny
                sty mapindex
                tay
                lda BLOCKCOLS,y
                sta drsc_lda2+1
                lda blktbllo,y
                clc
                adc scrolltemp1
                sta drsc_lda+1
                lda blktblhi,y
                sta drsc_lda+2
                ldy blockx
drsc_lda:       lda $1000,y
drsc_sta:       sta SCREEN1,x
drsc_lda2:      lda #$00
drsc_sta2:      sta COLORS,x
                inx
                cpx #39
                beq drsc_ready
                iny
                cpy #4
                bne drsc_lda
drsc_getblock:  ldy mapindex
                inc mapindex
                lda (maptemplo),y
                tay
                lda BLOCKCOLS,y
                sta drsc_lda2+1
                lda blktbllo,y
                clc
                adc scrolltemp1
                sta drsc_lda+1
                lda blktblhi,y
                sta drsc_lda+2
                ldy #0
                jmp drsc_lda
drsc_ready:     clc
                lda drsc_sta+1
                adc #40
                sta drsc_sta+1
                bcc drsc_notover1
                inc drsc_sta+2
                clc
drsc_notover1:  lda drsc_sta2+1
                adc #40
                sta drsc_sta2+1
                bcc drsc_notover2
                inc drsc_sta2+2
                clc
drsc_notover2:  lda scrolltemp1
                adc #4
                sta scrolltemp1
                cmp #16
                bcc drsc_notover3
                lda #0
                sta scrolltemp1
                clc
                lda maptemplo
                adc mapsx
                sta maptemplo
                bcc drsc_notover3
                inc maptemphi
drsc_notover3:  dec scrolltemp2
                beq drsc_alldone
                jmp drsc_rowloop
drsc_alldone:   jmp is_clear                    ;In the end, reset scrolling


;Ŀ
;GETCHARPOS                                                                   
;                                                                             
;Finds out an actor's blockmap position, for background collisions.           
;This routine is relatively slow.                                             
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

getcharpos:     lda shpyh,x
                sta scrolltemp2
                lda shpyl,x                     ;Divide Y by eight
                lsr scrolltemp2
                ror
                lsr scrolltemp2
                ror
                lsr scrolltemp2
                ror
                sta scrolltemp1                 ;Store the char-position
                and #3
                sta charblocky
                lda scrolltemp1
                ldy scrolltemp2
                lsr                             ;Divide char-position by four
                lsr                             ;to get map-position
                clc
                adc rowhitbl,y                  ;(add MSB)
                sta charmapy
                lda shpxh,x
                sta scrolltemp2
                lda shpxl,x                     ;Same for X-coord
                lsr scrolltemp2
                ror
                lsr scrolltemp2
                ror
                lsr scrolltemp2
                ror
                sta scrolltemp1
                and #3
                sta charblockx
                lda scrolltemp1
                ldy scrolltemp2
                lsr
                lsr
                clc
                adc rowhitbl,y
                sta charmapx
                rts


;Ŀ
;Routines to move the background collision checking point                     
;Observe: for example 4 chars move is fastest by modifying charmapx or        
;charmapy directly                                                            
;                                                                             
;Parameters: Y:amount of move (max. 4)                                        
;Returns: -                                                                   
;Modifies: A                                                                  
;

movecharposright:
                tya
                clc
                adc charblockx
                cmp #4
                bcc mcpr_notover
                sbc #4
                inc charmapx
mcpr_notover:   sta charblockx
                rts

movecharposleft:sty scrolltemp1
                lda charblockx
                sec
                sbc scrolltemp1
                bcs mcpl_notover
                adc #4
                dec charmapx
mcpl_notover:   sta charblockx
                rts

movecharposdown:tya
                clc
                adc charblocky
                cmp #4
                bcc mcpd_notover
                sbc #4
                inc charmapy
mcpd_notover:   sta charblocky
                rts

movecharposup:  sty scrolltemp1
                lda charblocky
                sec
                sbc scrolltemp1
                bcs mcpu_notover
                adc #4
                dec charmapy
mcpu_notover:   sta charblocky
                rts

;Ŀ
;Routines to move the checking spot by 1 char                                 
;                                                                             
;Parameters: -                                                                
;Returns: -
;Modifies: A                                                                  
;

movecharposright1:
                inc charblockx
                lda charblockx
                cmp #4
                bcs mcpr1_over
                rts
mcpr1_over:     lda #0
                sta charblockx
                inc charmapx
                rts

movecharposleft1:
                dec charblockx
                bmi mcpl1_over
                rts
mcpl1_over:     lda #3
                sta charblockx
                dec charmapx
                rts

movecharposdown1:
                inc charblocky
                lda charblocky
                cmp #4
                bcs mcpd1_over
                rts
mcpd1_over:     lda #0
                sta charblocky
                inc charmapy
                rts

movecharposup1: dec charblocky
                bmi mcpu1_over
                rts
mcpu1_over:     lda #3
                sta charblocky
                dec charmapy
                rts

;Ŀ
;GETCHARINFO                                                                  
;                                                                             
;Gets the char-info from the checking spot. This routine is quite fast!       
;Tm rutiini on kohtalaisen nopea!                                           
;                                                                             
;Parameters: -                                                                
;Returns: A:char-info (background collision info)                             
;Modifies: A,Y                                                                
;

getcharinfo:    ldy charmapy
                lda maptbllo,y
                sta scrolltemp1
                lda maptblhi,y
                sta scrolltemp2
                ldy charmapx
                lda (scrolltemp1),y              ;Get block
                tay
                lda blktbllo,y
                sta scrolltemp1
                lda blktblhi,y
                sta scrolltemp2
                lda charblocky
                asl
                asl
                adc charblockx
                tay
                lda (scrolltemp1),y             ;Get char from the block
                tay
                lda CHARINFO,y                  ;Get char-info
                rts


