;ͻ
;Sound effect routines                                                        
;ͼ

SFX_NONE        = 0
SFX_FOOTSTEP    = 1
SFX_JUMP        = 2
SFX_SELECT      = 3
SFX_THROW       = 4
SFX_FLAME       = 5
SFX_PISTOL      = 6
SFX_PLASMA      = 7
SFX_LASER       = 8
SFX_HIT         = 9
SFX_EXPLOSION   = 10
SFX_TANK        = 11

;Ŀ
;PLAYSFX                                                                      
;                                                                             
;Plays an effect if sound effects are chosen.                                 
;                                                                             
;Parameters: A:Sound number                                                   
;Returns: -                                                                   
;Modifies: -                                                                  
;

playsfx:        pha
                lda sfxmode
                beq psfx_skip2
                lda playmusic+1
                bpl psfx_skip2
                pla
                stx psfx_xback+1
                tax
                lda sfxpritbl-1,x               ;Check priority
                cmp sfxpri
                bcc psfx_xback
                sta sfxpri
                stx sfx+1
psfx_xback:     ldx #$00
                rts
psfx_skip2:     pla
                rts

;Ŀ
;PLAYTUNE                                                                     
;                                                                             
;Plays a song. Ingame songs (2,3,4,5,6) are not played if sound effects are   
;chosen.                                                                      
;                                                                             
;Parameters: A:song number                                                    
;Returns: -                                                                   
;Modifies: A                                                                  
;

playtune:       stx pt_restx+1
                ldx #MUS_STOP
                stx lastgametune
                cmp #MUS_CITY
                bcc pt_init
                cmp #MUS_DEATH
                bcs pt_init
                ldx sfxmode
                bne pt_nomusic
                sta lastgametune
pt_init:        ldx #$00                ;Reset soundeffect-system
                stx sfx+1
                stx sfxlen
                stx sfxpri
                sta playmusic+1         ;And set songnumber to be initted
pt_restx:       ldx #$00
pt_stopped:     rts
pt_nomusic:     sta lastgametune
                lda playmusic+1
                bmi pt_stopped
                lda #MUS_STOP
                jmp pt_init

;Ŀ
;LEVELTUNE                                                                    
;                                                                             
;Plays music associated with a level.                                         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

leveltune:      lda bossfight
                beq lt_noboss
                lda #MUS_BOSSFIGHT
                bne lt_check
lt_noboss:      ldx level
                lda lvltune,x
lt_check:       cmp lastgametune
                bne lt_different
                rts
lt_different:   jmp playtune

leveltune_nocheck:
                lda bossfight
                beq ltn_noboss
                lda #MUS_BOSSFIGHT
                jmp playtune
ltn_noboss:     ldx level
                lda lvltune,x
                jmp playtune

;Ŀ
;SELECTSFX                                                                    
;                                                                             
;Toggles sound effects on/off is S-key is pressed.                            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

selectsfx:      lda keyval
                cmp #KEY_S
                bne ts_nokey
togglesfx:      lda sfxmode
                eor #$01
                sta sfxmode
                bne ts_on
                lda lastgametune
                sta playmusic+1
                lda #MSG_MUSICON
                ldx #50
                jsr newmsg
                rts
ts_on:          lda #MUS_STOP
                jsr pt_init
ts_nostop:      lda #MSG_SFXON
                ldx #50
                jsr newmsg
ts_nokey:       rts

;Ŀ
;DOSFX                                                                        
;                                                                             
;Plays one "frame" of music/sounds. Called from the raster interrupt.         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

dosfx:          lda playmusic+1
                bmi sfx
                jmp playmusic
sfx:            lda #$00                ;Sound initialization?
                beq sfx_noinit
                jmp sfx_init
sfx_noinit:     tax
                lda sfxlen              ;Duration left?
                bne sfx_notdone
                rts
sfx_notdone:    dec sfxlen
                bne sfx_chloop
                stx sfxpri
sfx_chloop:     ldy chsndnum,x
                inc chfreqtime,x
                clc
                lda chfreqdir,x         ;Use pitch-modification 1 or 2?
                bne sfx_freqdown
                lda chfreqhi,x
                adc sfxadd1tbl,y
                sta chfreqhi,x
                lda chfreqtime,x
                cmp sfxtime1tbl,y
                bne sfx_freqdone
sfx_dirchg:     lda chfreqdir,x
                eor #$ff
                sta chfreqdir,x
                lda #$00
                sta chfreqtime,x
                beq sfx_freqdone
sfx_freqdown:   lda chfreqhi,x
                adc sfxadd2tbl,y
                sta chfreqhi,x
                lda chfreqtime,x
                cmp sfxtime2tbl,y
                beq sfx_dirchg
sfx_freqdone:   lda chgatetime,x        ;Time to clear gatebit?
                beq sfx_nogate
                dec chgatetime,x
                bne sfx_nogate
                lda chwave,x
                and #$fe
                sta chwave,x
sfx_nogate:     ldy chregnum,x
                lda #$00
                sta $d400,y
                lda chfreqhi,x
                sta $d401,y
                lda chwave,x
                sta $d404,y
                lda chad,x
                sta $d405,y
                lda chsr,x
                sta $d406,y
                inx                     ;All channels done?
                cpx #$03
                beq sfx_done
                jmp sfx_chloop
sfx_done:       lda #$0f
                sta $d418
                rts

sfx_init:       tay
                dey
                sty sfxtmp
                lda sfxlentbl,y
                sta sfxlen
                ldx #$00
                stx sfx+1
                stx $d417               ;Init filter
                lda sfxtmp
                asl
                adc sfxtmp
                tay
sfx_initloop:   lda sfxfreqtbl,y        ;Init channel specific things
                sta chfreqhi,x
                lda sfxwavetbl,y
                sta chwave,x
                lda sfxgatetimetbl,y
                sta chgatetime,x
                lda sfxadtbl,y
                sta chad,x
                lda sfxsrtbl,y
                sta chsr,x
                tya
                sta chsndnum,x
                lda #$00
                sta chfreqtime,x
                sta chfreqdir,x
                stx sfxtmp
                lda chregnum,x
                tax
                lda #$00
                sta $d402,x
                sta $d404,x
                sta $d405,x             ;Hard restart
                sta $d406,x
                lda sfxplshitbl,y
                sta $d403,x
                ldx sfxtmp
                inx
                iny
                cpx #$03
                bne sfx_initloop
                rts


