;ͻ
;General actor routines + leveldata routines                                  
;ͼ

;Ŀ
;COLLIDE                                                                      
;                                                                             
;Checks collision of two actors                                               
;                                                                             
;Parameters: X,Y:actor numbers                                                
;Returns: Carry=1 if collided                                                 
;Modifies: A                                                                  
;

collide:        stx shpnum1                     ;Store actor numbers
                sty shpnum2
                cpy shpnum1                     ;Return if same
                beq col_quit
                lda shpt,x                      ;Check that both actors are
                beq col_quit                    ;active, get their datastructure
                tax                             ;addresses (for sizes)
                lda shptbllo-1,x
                sta shpadrlo1
                lda shptblhi-1,x
                sta shpadrhi1
                lda shpt,y
                beq col_quit
                tax
                lda shptbllo-1,x
                sta shpadrlo2
                lda shptblhi-1,x
                sta shpadrhi2
                ldx shpnum1
                lda shpxl,x                     ;Subtract X-coords
                sec
                sbc shpxl,y
                sta ahi
                lda shpxh,x
                sbc shpxh,y
                bpl col_xnotneg                 ;If negative result, subtract
                lda shpxl,y                     ;the other way round
                sec
                sbc shpxl,x
                sta ahi
                lda shpxh,y
                sbc shpxh,x
col_xnotneg:    bne col_quit                    ;If highbyte > 0, actors too far
                ldy #5                          ;away to collide
                lda (shpadrlo1),y               ;Add X-sizes together
                clc
                adc (shpadrlo2),y
                ldy shpnum2
                cmp ahi                         ;Compare to result of subtraction
                bcc col_quit                    ;If less, no collision
                lda shpyl,x                     ;Do same for Y-coords
                sec
                sbc shpyl,y
                sta ahi
                lda shpyh,x
                sbc shpyh,y
                bmi col_yneg
                bne col_quit
                ldy #6
                lda (shpadrlo1),y
                clc
                iny
                adc (shpadrlo2),y
                ldy shpnum2
                cmp ahi
                rts
col_quit:       ldx shpnum1
                clc
                rts
col_yneg:       lda shpyl,y
                sec
                sbc shpyl,x
                sta ahi
                lda shpyh,y
                sbc shpyh,x
                bne col_quit
                ldy #6
                lda (shpadrlo2),y
                clc
                iny
                adc (shpadrlo1),y
                ldy shpnum2
                cmp ahi
                rts

;Ŀ
;ENEMYCOLL                                                                    
;                                                                             
;Checks player - enemy collisions.                                            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

enemycoll:      ldx #FIRSTNPC
ec1:            lda shpt,x                      ;Is it an enemy?
                cmp #FENEMYTYPE
                bcs ec2
ecnext:         inx
                cpx #LASTNPC+1
                bne ec1
                rts
ec2:            cmp #LENEMYTYPE+1
                bcs ecnext
                sec
                sbc #FENEMYTYPE
                tay
                lda shpp,x                      ;If an ememy has zero power
                beq ecnext                      ;(dying) it cannot do damage
                lda enemydamage,y
                beq ecnext
                sta alo
                ldy #PLAYER
                jsr collide
                bcc ecnext
                jsr damage                      ;Only one enemy causes damage
                rts                             ;at a time

;Ŀ
;PLRBULLETCOLL                                                                
;                                                                             
;Player bullet collisions to enemies.                                         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

EXPLTHRESHOLD   = 4
plrbulletcoll:  ldy #FIRSTNPC
pbc_loop:       lda shpt,y                      ;Is it an enemy?
                cmp #FENEMYTYPE
                bcc pbc_next
                cmp #LENEMYTYPE+1
                bcs pbc_next
                lda shpimm,y                    ;Enemy immortal?
                bne pbc_next
                lda shpp,y                      ;If health is zero, skip
                beq pbc_next
                ldx #FIRSTPLRBLT
                lda shpt,x
                cmp #SHP_BULLET
                bne pbc_no1
                jsr collide                     ;Check collision with bullet
                bcs pbc_damage
pbc_no1:        inx
                lda shpt,x
                cmp #SHP_BULLET
                bne pbc_no2
                jsr collide
                bcs pbc_damage
pbc_no2:        inx
                lda shpt,x
                cmp #SHP_BULLET
                bne pbc_next
                jsr collide
                bcs pbc_damage
pbc_next:       iny
                cpy #LASTNPC+1
                bne pbc_loop
                rts

pbc_damage:     sty scrolltemp1
                ldy shpf,x
                lda wpndamage,y                 ;Get amount of damage
                cmp #EXPLTHRESHOLD
                bcc pbcnoexpl2
                jsr blast
                jsr explodeshape
                jmp pbc_next
pbcnoexpl2:     sta ahi
                lda #$00
                sta shpf,x                      ;Initialize blood animation
                sta shpfd,x
                lda #SHP_BLOOD
                sta shpt,x
                ldy scrolltemp1
                jsr pbcdamage
                ldy scrolltemp1
                jmp pbc_next

pbcdamage:      lda #$03                        ;Flash enemy
                sta shpimm,y
                lda shpp,y                      ;And do damage
                sec
                sbc ahi
                sta shpp,y
                beq pbc_death
                bcs pbc_nodeath                 ;Died?
                lda #$00
                sta shpp,y
pbc_death:      lda shpt,y
                sec
                sbc #FENEMYTYPE
                sta pwr_enemycheck+1
                tax
                lda enemyscorelo,x              ;Give score to player, according
                sta alo                         ;to the enemy score table
                lda enemyscorehi,x
                sta ahi
                jsr addscore
                stx alo
                ldx shporg,y
                cpx #$ff
                beq pbc_noreappear
                lda lvlshpp,x                   ;If health in leveldata is
                bne pbc_noreappear              ;zero, the enemy always
                lda shpt,y                      ;reappears!
                sta lvlshpt,x
                lda #$ff
                sta shporg,y
pbc_noreappear: ldx alo
                lda enemydeathshp,x             ;Enemy's dead actor type
                sta shpt,y
                lda shpyl,y
                sec
                sbc enemydeathymod,x
                sta shpyl,y
                lda shpyh,y
                sbc #$00
                sta shpyh,y
                lda #$00
                sta shpf,y
                sta shpfd,y
                lda enemydeathsfx,x             ;Death sound effect
                beq pbc_boss
                jsr playsfx
                jsr powerup
pbc_boss:       rts
pbc_nodeath:    lda ahi
                cmp #EXPLTHRESHOLD
                bcs pbc_nosound
                ldy #SFX_HIT
                tya
                jsr playsfx
pbc_nosound:    rts

blast:          cpx #FIRSTPLRBLT                ;Checking radius damage of
                bcs blast2                      ;a grenade
                rts
blast2:         lda #SHP_BLAST
                sta shpt,x
                ldy #FIRSTNPC
blastloop:      lda shpt,y
                cmp #FENEMYTYPE
                bcc blastnext
                cmp #LENEMYTYPE+1
                bcs blastnext
                lda shpimm,y
                bne blastnext
                lda shpp,y
                beq blastnext
                jsr collide
                bcc blastnext
                lda #4
                sta ahi
                stx scrolltemp1
                sty scrolltemp2
                jsr pbcdamage
                ldx scrolltemp1
                ldy scrolltemp2
blastnext:      iny
                cpy #LASTNPC+1
                bcc blastloop
                rts

;Ŀ
;ENMBULLETCOLL                                                                
;                                                                             
;Checks enemy bullet collisions to player.                                    
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

enmbulletcoll:  lda shpt
                beq ebc_done
                cmp #SHP_PLRDEAD
                beq ebc_done
                ldy #$00
                ldx #FIRSTENMBLT
                lda shpt,x
                cmp #SHP_BULLET
                bne ebc_no1
                jsr collide
                bcs ebc_damage
ebc_no1:        inx
                lda shpt,x
                cmp #SHP_BULLET
                bne ebc_done
                jsr collide
                bcs ebc_damage
ebc_done:       rts

ebc_damage:     ldy shpf,x
                lda wpndamage,y
                sta alo
                lda #$00
                sta shpf,x
                sta shpfd,x
                lda #SHP_BLOOD
                sta shpt,x
                ldy #SFX_HIT
                lda alo
                cmp #EXPLTHRESHOLD
                bcc ebcnoexpl2
                ldy #SFX_EXPLOSION
                lda #SHP_EXPLOSION
                sta shpt,x
ebcnoexpl2:     tya
                jsr playsfx
                jsr damage
                rts

;Ŀ
;POWERUP                                                                      
;                                                                             
;Spawns a powerup item after an enemy dies.                                   
;                                                                             
;Parameters: Y:Enemy actor number                                             
;Returns: -                                                                   
;Modifies: A,X                                                                
;

powerup:        jsr random
                sec
                sbc #$c0
                bcs pwr_ok
                rts
pwr_ok:         lsr
                lsr
                lsr
                sta alo                                 ;Random type of
pwr_search:     ldx #PWRUP                              ;powerup
                lda shpt,x
                beq pwr_found
                rts
pwr_found:      sty pwr_yback+1
                lda shpxl,y
                sta shpxl,x
                lda shpxh,y
                sta shpxh,x
                lda shpyl,y
                sta shpyl,x
                lda shpyh,y
                sta shpyh,x
                lda #$ff
                sta shporg,x
                ldy alo
                cpy #6                                  ;Weapon?
                bcc pwr_noweapon
pwr_enemycheck: lda #0
                cmp #4                                  ;Nonhuman enemy?
                bcc pwr_noweapon                        ;Instead of giving
                ldy #2                                  ;rifle/grenade, give
pwr_noweapon:   lda powerupptbl,y                       ;minimum credits
                sta shpp,x
                lda powerupttbl,y
                sta shpf,x
                lda #$00
                sta shpimm,x
                lda #$01
                sta shpsy,x
                lda #SHP_ITEM
                sta shpt,x
pwr_yback:      ldy #$00
                rts

;Ŀ
;CLEARSHAPES                                                                  
;                                                                             
;Resets all actors, done at startup.                                          
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

clearshapes:    ldx #MAX_SHP-1
clrshploop:     lda #0
                sta shpt,x
                sta shpimm,x
                lda #$ff
                sta shporg,x
                dex
                bpl clrshploop
                rts

;Ŀ
;MOVESHAPES                                                                   
;                                                                             
;Calls the move routine of each actor.                                        
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

moveshapes:     ldx #MAX_SHP-1
mvsh_loop:      ldy shpt,x
                bne mvsh_notempty
                dex
                bpl mvsh_loop
                rts
mvsh_notempty:  lda movtbllo-1,y
                sta mvsh_jsr+1
                lda movtblhi-1,y
                sta mvsh_jsr+2
mvsh_jsr:       jsr $1000
                dex
                bpl mvsh_loop
                rts

;Ŀ
;ADDSHAPES                                                                    
;                                                                             
;Adds actors (that are close enough) from the leveldata to the screen         
;Checks only 16 actors each frame (out of 64) to reduce CPU load.             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

addshapes:
adsh_start:     ldx #$00
                lda mapx
                clc
                adc #10
                sta maptemplo
                lda mapy
                clc
                adc #6
                sta maptemphi
adsh_loop:      lda lvlshpt,x                           ;Actor type
                beq adsh_next                           ;0 -> empty
                lda lvlshpy,x                           ;Y-coord too low?
                cmp mapy
                bcc adsh_next
                cmp maptemphi                           ;Too high?
                bcs adsh_next
                lda lvlshpx,x                           ;Same for X-coord
                cmp mapx
                bcc adsh_next
                cmp maptemplo
                bcc adsh_search
                bne adsh_next
                lda blockx
                cmp #2
                bcs adsh_search
adsh_next:      inx
adsh_end:       cpx #$10
                bne adsh_loop
adsh_incr:      lda adsh_start+1
                clc
                adc #$10
                and #$3f
                sta adsh_start+1
                adc #$10
                sta adsh_end+1
                rts

adsh_search:    ldy #FIRSTNPC
                REPEAT LASTNPC-FIRSTNPC+1               ;Find an empty index
                lda shpt,y                              ;for the actor
                beq adsh_ok
                iny
                REPEND                                  ;If not found, exit
                jmp adsh_incr

adsh_ok:        lda #$00
                sta scrolltemp1
                sta scrolltemp2
                lda lvlshpt,x
                cmp #FENEMYTYPE
                bcc adsh_notenemy
                cmp #LENEMYTYPE+1
                bcs adsh_notenemy
                sta alo
                lda bossfight                           ;During a bossfight
                bne adsh_next                           ;ordinary enemies are
                lda alo                                 ;not added onscreen
                jmp adsh_enemy
adsh_notenemy:  lda lvlshpx,x
                asl
                rol scrolltemp1
                asl
                rol scrolltemp1
                asl
                rol scrolltemp1
                asl
                rol scrolltemp1
                asl
                rol scrolltemp1
                adc #16
                sta shpxl,y
                lda scrolltemp1
                adc #0
                sta shpxh,y
                lda lvlshpy,x
                asl
                rol scrolltemp2
                asl
                rol scrolltemp2
                asl
                rol scrolltemp2
                asl
                rol scrolltemp2
                asl
                rol scrolltemp2
                sta shpyl,y
                lda scrolltemp2
                sta shpyh,y
                jmp adsh_initialize

adsh_enemy:     sty alo
                stx ahi
                sec
                sbc #FENEMYTYPE
                tay
                lda enemypower,y                ;Get enemy health
                sta adsh_power+1
                lda enemyspawn,y                ;Get spawnmode
                ldy alo                         ;Exact?
                cmp #SPW_EXACT
                beq adsh_notenemy
                ldx alo
                sta adsh_cmp+1
                jsr random                      ;Random Y-coordinate
                and #$07
                tay
                lda spawnytbl,y
                clc
                adc mapy
                asl                             ;Multiply by 32
                rol scrolltemp2
                asl
                rol scrolltemp2
                asl
                rol scrolltemp2
                asl
                rol scrolltemp2
                asl
                rol scrolltemp2
                ldy adsh_cmp+1                  ;If spawning in air, lift off
                cpy #SPW_AIR                    ;the ground
                bne adsh_notair
                clc
                adc #$08
adsh_notair:    sta shpyl,x
                lda scrolltemp2
                sta shpyh,x
                ldy ahi
                lda scrposxl                    ;Take X-pos directly from the
                clc                             ;scrolling position
                adc #$08
                sta shpxl,x
                lda scrposxh
                adc #$00
                sta shpxh,x
                jsr random                      ;Left or right
                and #$01
                sta shpd,x
                beq adsh_left
                lda shpxl,x
                clc
                adc #<352
                sta shpxl,x
                lda shpxh,x
                adc #>352
                sta shpxh,x
adsh_left:      lda shpxh,x
                bmi adsh_notright
                jsr getcharpos
                jsr getcharinfo
                and #CI_WALK+CI_OBSTACLE
adsh_cmp:       cmp #$00                    ;Check for right kind of BG graphics
                bne adsh_notright               ;at spawn location
                jsr movecharposup1
                jsr getcharinfo
                ldy alo
                ldx ahi
                and #CI_OBSTACLE
                beq adsh_initialize
adsh_notright:  ldx ahi
                jmp adsh_next
adsh_initialize:txa                             ;Store leveldata origin index of
                sta shporg,y                    ;the actor
                lda lvlshpp,x                   ;Store health
                bne adsh_powerok
adsh_power:     lda #$00
adsh_powerok:   sta shpp,y
                lda lvlshpt,x                   ;Is it an item?
                bmi adsh_item
                sta shpt,y
                lda #$00
                sta lvlshpt,x
                sta shpsx,y
                sta shpsy,y
                sta shpimm,y
                sta shpf,y
                sta shpfd,y
                sta shpbits,y
                lda shpt,y
                cmp #SHP_BOSS0
                bcc adsh_noboss
                cmp #SHP_BOSS3+1
                bcs adsh_noboss
                lda #$01
                sta bossfight
                jsr leveltune
adsh_noboss:    jmp adsh_next
adsh_item:      and #$7f
                sta shpf,y
                lda #$01
                sta shpsy,y
                lda #SHP_ITEM
                sta shpt,y
                lda #$00
                sta shpimm,y
                sta lvlshpt,x
                jmp adsh_next

;Ŀ
;REMOVESHAPE                                                                  
;                                                                             
;Removes an actor from screen back to leveldata.                              
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

removeshape:    ldy shporg,x                    ;Does this actor have an origin
                cpy #$ff                        ;in the leveldata
                beq rshp_end
                lda lvlshpp,y                   ;If health in leveldata is 0,
                beq rshp_skippwr                ;do not store
                lda shpp,x                      ;If health of actor is 0, do
                beq rshp_end                    ;not return back to leveldata
                sta lvlshpp,y
rshp_skippwr:   lda shpt,x                      ;Is it an item?
                cmp #SHP_ITEM
                bne rshp_noitem
                lda shpf,x                      ;Item frame = item type
                ora #$80
                sta lvlshpt,y
rshp_back:      lda #$ff
                sta shporg,x
rshp_end:       lda #0
                sta shpt,x
                rts
rshp_noitem:    sta lvlshpt,y                   ;If not an item, store the
                jmp rshp_back                   ;actor type directly

;Ŀ
;REMOVESHAPES                                                                 
;                                                                             
;Removes all actors (except player) back to leveldata.                        
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

removeshapes:   ldx #MAX_SHP-1
rshps_loop:     jsr removeshape
                dex
                bne rshps_loop
                rts

;Ŀ
;PUTLEVELDATA                                                                 
;                                                                             
;Stores this level's leveldata from the workarea to "storage".                
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

putleveldata:   lda level
                clc
                adc #(>LEVELDATA)-1
                sta pld_storeadr+2
                ldx #$00
pld_loop:       lda lvlshpx,x
pld_storeadr:   sta LEVELDATA,x
                inx
                bne pld_loop
                rts

;Ŀ
;GETLEVELDATA                                                                 
;                                                                             
;Gets leveldata from the "storage" to the workarea.                           
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

getleveldata:   lda level
                clc
                adc #(>LEVELDATA)-1
                sta gld_loop+2
                ldx #$00
gld_loop:       lda LEVELDATA,x
                sta lvlshpx,x
                inx
                bne gld_loop
                rts

;Ŀ
;UPDATESHAPES                                                                 
;                                                                             
;Divides actors into sprites and defines also sprite amounts on every row     
;(for the sorting). Removes also off-screen actors!                           
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

updateshapes:   lda #$00
                sta numspr
                tax
upsh_loop:      ldy shpt,x
                bne upsh_notempty
                inx
                cpx #MAX_SHP
                bcc upsh_loop
                rts
upsh_removecheck:
                lda shpt,x
                cmp #SHP_PLRDEAD+1
                bcc upsh_dontremove
                cmp #SHP_BOSS3+1
                bcs upsh_remove
                cmp #SHP_BOSS0
                bcs upsh_dontremove
upsh_remove:    jsr removeshape
upsh_dontremove:inx
                cpx #MAX_SHP
                bcc upsh_loop
                rts

upsh_notempty:  stx shpindex
                lda shptbllo-1,y              ;Pointer to actor structure
                sta sptrlo
                lda shptblhi-1,y
                sta sptrhi
                lda shpxl,x                   ;Take scrolling into account
                sec                           ;(actors scroll with the
                sbc scrposxl                  ;background)
                sta tempshpxl
                lda shpxh,x
                sbc scrposxh
                sta tempshpxh
                lda shpyl,x
                sec
                sbc scrposyl
                sta tempshpyl
                lda shpyh,x
                sbc scrposyh
                sta tempshpyh
                lda tempshpyh                 ;Y MSB nonzero?
                bne upsh_removecheck
                lda tempshpyl                 ;Check LSB
                cmp #22
                bcc upsh_removecheck
                cmp #RASTER0LINE+32
                bcs upsh_removecheck
                lda tempshpxh                 ;X MSB < 0
                bmi upsh_removecheck
                beq upsh_norcheck
                cmp #$02
                bcs upsh_removecheck
upsh_checklow:  lda tempshpxl                 ;X MSB > 0, check LSB
                cmp #112
                bcs upsh_removecheck
upsh_norcheck:  lda shpimm,x                  ;Check for immortality flashing
                beq upsh_noimm
                dec shpimm,x
                and #$02
                beq upsh_noimm
                lda #$ff
upsh_noimm:     sta alo
                ldy #$00                      ;X-hotspot
                lda tempshpxl
                sec
                sbc (sptrlo),y
                sta tempshpxl
                bcs upsh_notover1
                dec tempshpxh
                sec
upsh_notover1:  iny                           ;Y-hotspot
                lda tempshpyl
                sbc (sptrlo),y
                sta tempshpyl
                bcs upsh_notover2
                dec tempshpyh

upsh_notover2:  iny                           ;Spriterow configuration
                lda (sptrlo),y
                tay
                cpy #SHP4x2                   ;Is it a boss enemy?
                bcc upsh_noboss
                lda bossdeath
                and #$01
                bne upsh_bossexp
upsh_noboss:    lda upsh_routtbllo,y          ;Get corresponding spriterow
                sta upsh_jump+1               ;routine address
                lda upsh_routtblhi,y
                sta upsh_jump+2
                lda shpf,x                    ;Actor's frame
                ldy shpd,x                    ;Actor's direction
                beq upsh_jump                 ;If left, add first "left" frame
                ldy #$04
                clc
                adc (sptrlo),y
upsh_jump:      jmp $0000                     ;Then jump to drawing the sprites

upsh_bossexp:   ldy #NBOSSEXP-1                 ;Boss enemy explosions
                ldx numspr
upsh_bossexploop:
                cpx #MAX_SPR
                beq upsh_bossdone
                lda bossexpf,y
                bmi upsh_bossskip
                sta sprf,x
                lda tempshpxl
                clc
                adc bossexpxl,y
                sta sprx,x
                lda tempshpxh
                adc #$00
                sta sprxh,x
                lda tempshpyl
                clc
                adc bossexpyl,y
                sta spry,x
                lda #$00
                sta sprfls,x
                inx
                stx numspr
upsh_bossskip:  dey
                bpl upsh_bossexploop
upsh_bossdone:  jmp upsh_nextshape

upsh_1x1:       clc                           ;Frame pointer in Y-register
                adc #$08
                tay
                ldx numspr
                cpx #MAX_SPR
                bcs upsh_nextshape
                inc numspr
                lda alo                       ;Sprite flashing
                sta sprfls,x
                lda tempshpyl                 ;Y-coord
                sta spry,x
                lda tempshpxl                 ;X-coord
                sta sprx,x
                lda tempshpxh
                sta sprxh,x
                lda (sptrlo),y                ;Frame
                sta sprf,x
upsh_nextshape: ldx shpindex
                inx
                cpx #MAX_SHP
                bcs upsh_alldone
                jmp upsh_loop
upsh_alldone:   rts

upsh_1x2:       asl
                adc #$08
                tay
                ldx numspr
                cpx #MAX_SPR-1
                bcs upsh_nextshape
                inc numspr
                inc numspr
                lda alo
                sta sprfls,x
                sta sprfls+1,x
                lda tempshpyl
                sta spry,x
                clc
                adc #21
                sta spry+1,x
                lda tempshpxl
                sta sprx,x
                sta sprx+1,x
                lda tempshpxh
                sta sprxh,x
                sta sprxh+1,x
                jsr upsh_loadframes
                jmp upsh_nextshape

upsh_2x1:       asl
                adc #$08
                tay
                ldx numspr
                cpx #MAX_SPR-1
                bcs upsh_nextshape
                inc numspr
                inc numspr
                lda alo
                sta sprfls,x
                sta sprfls+1,x
                lda tempshpyl
                sta spry,x
                sta spry+1,x
                lda tempshpxl
                sta sprx,x
                clc
                adc #24
                sta sprx+1,x
                lda tempshpxh
                sta sprxh,x
                adc #0
                sta sprxh+1,x
                jsr upsh_loadframes
                jmp upsh_nextshape

upsh_2x2:       asl
                asl
                adc #$08
                tay
                ldx numspr
                cpx #MAX_SPR-3
                bcc upsh_2x2ok
                jmp upsh_nextshape
upsh_2x2ok:     lda numspr
                adc #$04
                sta numspr
                lda alo
                sta sprfls,x
                sta sprfls+1,x
                sta sprfls+2,x
                sta sprfls+3,x
                lda tempshpyl
                sta spry,x
                sta spry+1,x
                clc
                adc #21
                sta spry+2,x
                sta spry+3,x
                lda tempshpxl
                sta sprx,x
                sta sprx+2,x
                clc
                adc #24
                sta sprx+1,x
                sta sprx+3,x
                lda tempshpxh
                sta sprxh,x
                sta sprxh+2,x
                adc #0
                sta sprxh+1,x
                sta sprxh+3,x
                jsr upsh_loadframes
                jmp upsh_nextshape

upsh_4x2:       asl
                asl
                asl
                adc #$08
                tay
                ldx numspr
                cpx #MAX_SPR-7
                bcc upsh_4x2ok
                jmp upsh_nextshape
upsh_4x2ok:     lda numspr
                adc #$08
                sta numspr
                lda alo
                sta sprfls,x
                sta sprfls+1,x
                sta sprfls+2,x
                sta sprfls+3,x
                sta sprfls+4,x
                sta sprfls+5,x
                sta sprfls+6,x
                sta sprfls+7,x
                lda tempshpyl
                sta spry,x
                sta spry+1,x
                sta spry+2,x
                sta spry+3,x
                clc
                adc #21
                sta spry+4,x
                sta spry+5,x
                sta spry+6,x
                sta spry+7,x
                lda tempshpxl
                sta sprx,x
                sta sprx+4,x
                clc
                adc #24
                sta sprx+1,x
                sta sprx+5,x
                lda tempshpxh
                sta sprxh,x
                sta sprxh+4,x
                adc #0
                sta sprxh+1,x
                sta sprxh+5,x
                lda sprx+1,x
                clc
                adc #24
                sta sprx+2,x
                sta sprx+6,x
                lda sprxh+1,x
                adc #0
                sta sprxh+2,x
                sta sprxh+6,x
                lda sprx+2,x
                clc
                adc #24
                sta sprx+3,x
                sta sprx+7,x
                lda sprxh+2,x
                adc #0
                sta sprxh+3,x
                sta sprxh+7,x
                jsr upsh_loadframes
                jmp upsh_nextshape

upsh_3x3:       sta ahi
                asl
                asl
                asl
                adc ahi
                adc #$08
                tay
                ldx numspr
                cpx #MAX_SPR-8
                bcc upsh_3x3ok
                jmp upsh_nextshape
upsh_3x3ok:     lda numspr
                adc #$09
                sta numspr
                lda alo
                sta sprfls,x
                sta sprfls+1,x
                sta sprfls+2,x
                sta sprfls+3,x
                sta sprfls+4,x
                sta sprfls+5,x
                sta sprfls+6,x
                sta sprfls+7,x
                sta sprfls+8,x
                lda tempshpyl
                sta spry,x
                sta spry+1,x
                sta spry+2,x
                clc
                adc #21
                sta spry+3,x
                sta spry+4,x
                sta spry+5,x
                adc #21
                sta spry+6,x
                sta spry+7,x
                sta spry+8,x
                lda tempshpxl
                sta sprx,x
                sta sprx+3,x
                sta sprx+6,x
                clc
                adc #24
                sta sprx+1,x
                sta sprx+4,x
                sta sprx+7,x
                lda tempshpxh
                sta sprxh,x
                sta sprxh+3,x
                sta sprxh+6,x
                adc #0
                sta sprxh+1,x
                sta sprxh+4,x
                sta sprxh+7,x
                lda sprx+1,x
                clc
                adc #24
                sta sprx+2,x
                sta sprx+5,x
                sta sprx+8,x
                lda sprxh+1,x
                adc #0
                sta sprxh+2,x
                sta sprxh+5,x
                sta sprxh+8,x
                jsr upsh_loadframes
                jmp upsh_nextshape

upsh_2x4:       asl
                asl
                asl
                adc #$08
                tay
                ldx numspr
                cpx #MAX_SPR-7
                bcc upsh_2x4ok
                jmp upsh_nextshape
upsh_2x4ok:     lda numspr
                adc #$08
                sta numspr
                lda alo
                sta sprfls,x
                sta sprfls+1,x
                sta sprfls+2,x
                sta sprfls+3,x
                sta sprfls+4,x
                sta sprfls+5,x
                sta sprfls+6,x
                sta sprfls+7,x
                lda tempshpyl
                sta spry,x
                sta spry+1,x
                clc
                adc #21
                sta spry+2,x
                sta spry+3,x
                adc #21
                sta spry+4,x
                sta spry+5,x
                adc #21
                sta spry+6,x
                sta spry+7,x
                lda tempshpxl
                sta sprx,x
                sta sprx+2,x
                sta sprx+4,x
                sta sprx+6,x
                clc
                adc #24
                sta sprx+1,x
                sta sprx+3,x
                sta sprx+5,x
                sta sprx+7,x
                lda tempshpxh
                sta sprxh,x
                sta sprxh+2,x
                sta sprxh+4,x
                sta sprxh+6,x
                adc #0
                sta sprxh+1,x
                sta sprxh+3,x
                sta sprxh+5,x
                sta sprxh+7,x
                jsr upsh_loadframes
                jmp upsh_nextshape

upsh_loadframes:lda (sptrlo),y
                sta sprf,x
                iny
                inx
                cpx numspr
                bcc upsh_loadframes
                rts


